@@ -7,7 +7,7 @@ import (
 	"math/big"
 
 	"github.com/consensys/gnark-crypto/ecc"
-	"github.com/consensys/gnark-crypto/ecc/bls12-381/fr"
+	"github.com/consensys/gnark-crypto/ecc/bls12-377/fr"
 )
 
 // generator of the curve
@@ -16,7 +16,7 @@ var xGen big.Int
 var glvBasis ecc.Lattice
 
 func init() {
-	xGen.SetString("-15132376222941642752", 10)
+	xGen.SetString("9586122913090633729", 10)
 	_r := fr.Modulus()
 	ecc.PrecomputeLattice(_r, &xGen, &glvBasis)
 }
