@@ -1,13 +1,13 @@
 // Copyright 2020-2025 Consensys Software Inc.
 // Licensed under the Apache License, Version 2.0. See the LICENSE file for details.
 
-package bls12381
+package bls12377
 
 import (
 	"errors"
 
-	"github.com/consensys/gnark-crypto/ecc/bls12-381/fp"
-	"github.com/consensys/gnark-crypto/ecc/bls12-381/internal/fptower"
+	"github.com/consensys/gnark-crypto/ecc/bls12-377/fp"
+	"github.com/consensys/gnark-crypto/ecc/bls12-377/internal/fptower"
 )
 
 // GT target group of the pairing
@@ -50,6 +50,7 @@ func PairingCheck(P []G1Affine, Q []G2Affine) (bool, error) {
 // we use instead d=s ⋅ (p⁶-1)(p²+1)(p⁴ - p² +1)/r
 // where s is the cofactor 3 (Hayashida et al.)
 func FinalExponentiation(z *GT, _z ...*GT) GT {
+
 	var result GT
 	result.Set(z)
 
@@ -77,7 +78,7 @@ func FinalExponentiation(z *GT, _z ...*GT) GT {
 	// Daiki Hayashida, Kenichiro Hayasaka and Tadanori Teruya
 	// https://eprint.iacr.org/2020/875.pdf
 	t[0].CyclotomicSquare(&result)
-	t[1].ExptHalf(&t[0])
+	t[1].Expt(&result)
 	t[2].InverseUnitary(&result)
 	t[1].Mul(&t[1], &t[2])
 	t[2].Expt(&t[1])
@@ -136,50 +137,45 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 	if n >= 1 {
 		// i = 62, separately to avoid an E12 Square
 		// (Square(res) = 1² = 1)
-		// LoopCounter[62] = 1
-		// k = 0, separately to avoid MulBy014 (res × ℓ)
+		// LoopCounter[62] = 0
+		// k = 0, separately to avoid MulBy034 (res × ℓ)
 		// (assign line to res)
 
 		// qProj[0] ← 2qProj[0] and l1 the tangent ℓ passing 2qProj[0]
 		qProj[0].doubleStep(&l1)
 		// line evaluation at P[0] (assign)
-		result.C0.B0.Set(&l1.r0)
-		result.C0.B1.MulByElement(&l1.r1, &p[0].X)
-		result.C1.B1.MulByElement(&l1.r2, &p[0].Y)
+		result.C0.B0.MulByElement(&l1.r0, &p[0].Y)
+		result.C1.B0.MulByElement(&l1.r1, &p[0].X)
+		result.C1.B1.Set(&l1.r2)
+	}
 
-		// qProj[0] ← qProj[0]+Q[0] and
-		// l2 the line ℓ passing qProj[0] and Q[0]
-		qProj[0].addMixedStep(&l2, &q[0])
-		// line evaluation at P[0] (assign)
-		l2.r1.MulByElement(&l2.r1, &p[0].X)
-		l2.r2.MulByElement(&l2.r2, &p[0].Y)
+	if n >= 2 {
+		// k = 1, separately to avoid MulBy034 (res × ℓ)
+		// (res is also a line at this point, so we use Mul034By034 ℓ × ℓ)
+
+		// qProj[1] ← 2qProj[1] and l1 the tangent ℓ passing 2qProj[1]
+		qProj[1].doubleStep(&l1)
+		// line evaluation at P[1]
+		l1.r0.MulByElement(&l1.r0, &p[1].Y)
+		l1.r1.MulByElement(&l1.r1, &p[1].X)
 		// ℓ × res
-		prodLines = fptower.Mul014By014(&l2.r0, &l2.r1, &l2.r2, &result.C0.B0, &result.C0.B1, &result.C1.B1)
+		prodLines = fptower.Mul034By034(&l1.r0, &l1.r1, &l1.r2, &result.C0.B0, &result.C1.B0, &result.C1.B1)
 		result.C0.B0 = prodLines[0]
 		result.C0.B1 = prodLines[1]
 		result.C0.B2 = prodLines[2]
-		result.C1.B1 = prodLines[3]
-		result.C1.B2 = prodLines[4]
+		result.C1.B0 = prodLines[3]
+		result.C1.B1 = prodLines[4]
 	}
 
-	// k >= 1
-	for k := 1; k < n; k++ {
+	// k >= 2
+	for k := 2; k < n; k++ {
 		// qProj[k] ← 2qProj[k] and l1 the tangent ℓ passing 2qProj[k]
 		qProj[k].doubleStep(&l1)
 		// line evaluation at P[k]
+		l1.r0.MulByElement(&l1.r0, &p[k].Y)
 		l1.r1.MulByElement(&l1.r1, &p[k].X)
-		l1.r2.MulByElement(&l1.r2, &p[k].Y)
-
-		// qProj[k] ← qProj[k]+Q[k] and
-		// l2 the line ℓ passing qProj[k] and Q[k]
-		qProj[k].addMixedStep(&l2, &q[k])
-		// line evaluation at P[k]
-		l2.r1.MulByElement(&l2.r1, &p[k].X)
-		l2.r2.MulByElement(&l2.r2, &p[k].Y)
-		// ℓ × ℓ
-		prodLines = fptower.Mul014By014(&l2.r0, &l2.r1, &l2.r2, &l1.r0, &l1.r1, &l1.r2)
-		// (ℓ × ℓ) × result
-		result.MulBy01245(&prodLines)
+		// ℓ × res
+		result.MulBy034(&l1.r0, &l1.r1, &l1.r2)
 	}
 
 	// i <= 61
@@ -192,49 +188,55 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 			// qProj[k] ← 2qProj[k] and l1 the tangent ℓ passing 2qProj[k]
 			qProj[k].doubleStep(&l1)
 			// line evaluation at P[k]
+			l1.r0.MulByElement(&l1.r0, &p[k].Y)
 			l1.r1.MulByElement(&l1.r1, &p[k].X)
-			l1.r2.MulByElement(&l1.r2, &p[k].Y)
 
 			if LoopCounter[i] == 0 {
 				// ℓ × res
-				result.MulBy014(&l1.r0, &l1.r1, &l1.r2)
+				result.MulBy034(&l1.r0, &l1.r1, &l1.r2)
 			} else {
 				// qProj[k] ← qProj[k]+Q[k] and
 				// l2 the line ℓ passing qProj[k] and Q[k]
 				qProj[k].addMixedStep(&l2, &q[k])
 				// line evaluation at P[k]
+				l2.r0.MulByElement(&l2.r0, &p[k].Y)
 				l2.r1.MulByElement(&l2.r1, &p[k].X)
-				l2.r2.MulByElement(&l2.r2, &p[k].Y)
 				// ℓ × ℓ
-				prodLines = fptower.Mul014By014(&l2.r0, &l2.r1, &l2.r2, &l1.r0, &l1.r1, &l1.r2)
-				// (ℓ × ℓ) × result
-				result.MulBy01245(&prodLines)
+				prodLines = fptower.Mul034By034(&l1.r0, &l1.r1, &l1.r2, &l2.r0, &l2.r1, &l2.r2)
+				// (ℓ × ℓ) × res
+				result.MulBy01234(&prodLines)
 			}
 		}
+
 	}
 
-	// i = 0, separately to avoid a point doubling
-	// LoopCounter[0] = 0
+	// i = 0, separately to avoid a point addition
+	// LoopCounter[0] = 1
 	result.Square(&result)
 	for k := 0; k < n; k++ {
-		// l1 the tangent ℓ passing 2qProj[k]
-		qProj[k].tangentLine(&l1)
+		// qProj[k] ← 2qProj[k] and l1 the tangent ℓ passing 2qProj[k]
+		qProj[k].doubleStep(&l1)
 		// line evaluation at P[k]
+		l1.r0.MulByElement(&l1.r0, &p[k].Y)
 		l1.r1.MulByElement(&l1.r1, &p[k].X)
-		l1.r2.MulByElement(&l1.r2, &p[k].Y)
-		// ℓ × result
-		result.MulBy014(&l1.r0, &l1.r1, &l1.r2)
-	}
 
-	// negative x₀
-	result.Conjugate(&result)
+		// l2 the line passing qProj[k] and Q
+		qProj[k].lineCompute(&l2, &q[k])
+		// line evaluation at P[k]
+		l2.r0.MulByElement(&l2.r0, &p[k].Y)
+		l2.r1.MulByElement(&l2.r1, &p[k].X)
+		// ℓ × ℓ
+		prodLines = fptower.Mul034By034(&l1.r0, &l1.r1, &l1.r2, &l2.r0, &l2.r1, &l2.r2)
+		// (ℓ × ℓ) × res
+		result.MulBy01234(&prodLines)
+	}
 
 	return result, nil
 }
 
 // doubleStep doubles a point in Homogenous projective coordinates, and evaluates the line in Miller loop
 // https://eprint.iacr.org/2013/722.pdf (Section 4.3)
-func (p *g2Proj) doubleStep(l *lineEvaluation) {
+func (p *g2Proj) doubleStep(evaluations *lineEvaluation) {
 
 	// get some Element from our pool
 	var t1, A, B, C, D, E, EE, F, G, H, I, J, K fptower.E2
@@ -267,16 +269,15 @@ func (p *g2Proj) doubleStep(l *lineEvaluation) {
 	p.z.Mul(&B, &H)
 
 	// Line evaluation
-	l.r0.Set(&I)
-	l.r1.Double(&J).
-		Add(&l.r1, &J)
-	l.r2.Neg(&H)
-
+	evaluations.r0.Neg(&H)
+	evaluations.r1.Double(&J).
+		Add(&evaluations.r1, &J)
+	evaluations.r2.Set(&I)
 }
 
 // addMixedStep point addition in Mixed Homogenous projective and Affine coordinates
 // https://eprint.iacr.org/2013/722.pdf (Section 4.3)
-func (p *g2Proj) addMixedStep(l *lineEvaluation, a *G2Affine) {
+func (p *g2Proj) addMixedStep(evaluations *lineEvaluation, a *G2Affine) {
 
 	// get some Element from our pool
 	var Y2Z1, X2Z1, O, L, C, D, E, F, G, H, t0, t1, t2, J fptower.E2
@@ -306,34 +307,29 @@ func (p *g2Proj) addMixedStep(l *lineEvaluation, a *G2Affine) {
 		Sub(&J, &t2)
 
 	// Line evaluation
-	l.r0.Set(&J)
-	l.r1.Neg(&O)
-	l.r2.Set(&L)
+	evaluations.r0.Set(&L)
+	evaluations.r1.Neg(&O)
+	evaluations.r2.Set(&J)
 }
 
-// tangentCompute computes the tangent through [2]p in Homogenous projective coordinates.
-// It does not compute the resulting point [2]p.
-func (p *g2Proj) tangentLine(l *lineEvaluation) {
+// lineCompute computes the line through p in Homogenous projective coordinates
+// and a in affine coordinates. It does not compute the resulting point p+a.
+func (p *g2Proj) lineCompute(evaluations *lineEvaluation, a *G2Affine) {
 
 	// get some Element from our pool
-	var t1, B, C, D, E, H, I, J fptower.E2
-	B.Square(&p.y)
-	C.Square(&p.z)
-	D.Double(&C).
-		Add(&D, &C)
-	E.MulBybTwistCurveCoeff(&D)
-	H.Add(&p.y, &p.z).
-		Square(&H)
-	t1.Add(&B, &C)
-	H.Sub(&H, &t1)
-	I.Sub(&E, &B)
-	J.Square(&p.x)
+	var Y2Z1, X2Z1, O, L, t2, J fptower.E2
+	Y2Z1.Mul(&a.Y, &p.z)
+	O.Sub(&p.y, &Y2Z1)
+	X2Z1.Mul(&a.X, &p.z)
+	L.Sub(&p.x, &X2Z1)
+	t2.Mul(&L, &a.Y)
+	J.Mul(&a.X, &O).
+		Sub(&J, &t2)
 
 	// Line evaluation
-	l.r0.Set(&I)
-	l.r1.Double(&J).
-		Add(&l.r1, &J)
-	l.r2.Neg(&H)
+	evaluations.r0.Set(&L)
+	evaluations.r1.Neg(&O)
+	evaluations.r2.Set(&J)
 }
 
 // ----------------------
@@ -375,11 +371,8 @@ func PairingCheckFixedQ(P []G1Affine, lines [][2][len(LoopCounter) - 1]LineEvalu
 func PrecomputeLines(Q G2Affine) (PrecomputedLines [2][len(LoopCounter) - 1]LineEvaluationAff) {
 	var accQ G2Affine
 	accQ.Set(&Q)
-	n := len(LoopCounter)
-	// i = n - 2
-	accQ.doubleStep(&PrecomputedLines[0][n-2])
-	accQ.addStep(&PrecomputedLines[1][n-2], &Q)
-	for i := n - 3; i >= 0; i-- {
+
+	for i := len(LoopCounter) - 2; i >= 0; i-- {
 		if LoopCounter[i] == 0 {
 			accQ.doubleStep(&PrecomputedLines[0][i])
 		} else {
@@ -399,18 +392,14 @@ func MillerLoopFixedQ(P []G1Affine, lines [][2][len(LoopCounter) - 1]LineEvaluat
 
 	// no need to filter infinity points:
 	// 		1. if Pᵢ=(0,0) then -x/y=1/y=0 by gnark-crypto convention and so
-	// 		lines R0 and R1 are 0. It happens that result will stay, through
-	// 		the Miller loop, in 𝔽p⁶ because MulBy01(0,0,1),
-	// 		Mul01By01(0,0,1,0,0,1) and MulBy01245 set result.C0 to 0. At the
-	// 		end result will be in a proper subgroup of Fp¹² so it be reduced to
-	// 		1 in FinalExponentiation.
-	//
-	//      and/or
+	// 		lines R0 and R1 are 0. At the end it happens that result will stay
+	// 		1 through the Miller loop because MulBy34(1,0,0)==1
+	// 		Mul34By34(1,0,0,1,0,0)==1 and MulBy01234(1,0,0,0,0)==1.
 	//
 	// 		2. if Qᵢ=(0,0) then PrecomputeLines(Qᵢ) will return lines R0 and R1
 	// 		that are 0 because of gnark-convention (*/0==0) in doubleStep and
-	// 		addStep. Similarly to Pᵢ=(0,0) it happens that result be 1
-	// 		after the FinalExponentiation.
+	// 		addStep. Similarly to Pᵢ=(0,0) it happens that result stays 1
+	// 		throughout the MillerLoop.
 
 	// precomputations
 	yInv := make([]fp.Element, n)
@@ -429,52 +418,93 @@ func MillerLoopFixedQ(P []G1Affine, lines [][2][len(LoopCounter) - 1]LineEvaluat
 	var prodLines [5]E2
 
 	// Compute ∏ᵢ { fᵢ_{x₀,Q}(P) }
-	for i := len(LoopCounter) - 2; i >= 0; i-- {
+	if n >= 1 {
+		// i = 62, separately to avoid an E12 Square
+		// (Square(res) = 1² = 1)
+		// LoopCounter[62] = 0
+		// k = 0, separately to avoid MulBy34 (res × ℓ)
+		// (assign line to res)
+
+		// line evaluation at P[0] (assign)
+		result.C1.B0.MulByElement(&lines[0][0][62].R0, &xNegOverY[0])
+		result.C1.B1.MulByElement(&lines[0][0][62].R1, &yInv[0])
+		// the coefficient which MulBy34 sets to 1 happens to be already 1 (result = 1)
+	}
+
+	if n >= 2 {
+		// k = 1, separately to avoid MulBy34 (res × ℓ)
+		// (res is also a line at this point, so we use Mul34By34 ℓ × ℓ)
+		// line evaluation at P[1]
+		lines[1][0][62].R0.MulByElement(&lines[1][0][62].R0, &xNegOverY[1])
+		lines[1][0][62].R1.MulByElement(&lines[1][0][62].R1, &yInv[1])
+		// ℓ × res
+		prodLines = fptower.Mul34By34(&lines[1][0][62].R0, &lines[1][0][62].R1, &result.C1.B0, &result.C1.B1)
+		result.C0.B0 = prodLines[0]
+		result.C0.B1 = prodLines[1]
+		result.C0.B2 = prodLines[2]
+		result.C1.B0 = prodLines[3]
+		result.C1.B1 = prodLines[4]
+	}
+
+	// k >= 2
+	for k := 2; k < n; k++ {
+		// line evaluation at P[k]
+		lines[k][0][62].R0.MulByElement(&lines[k][0][62].R0, &xNegOverY[k])
+		lines[k][0][62].R1.MulByElement(&lines[k][0][62].R1, &yInv[k])
+		// ℓ × res
+		result.MulBy34(
+			&lines[k][0][62].R0,
+			&lines[k][0][62].R1,
+		)
+	}
+
+	for i := len(LoopCounter) - 3; i >= 0; i-- {
 		// mutualize the square among n Miller loops
 		// (∏ᵢfᵢ)²
 		result.Square(&result)
 
 		for k := 0; k < n; k++ {
 			// line evaluation at P[k]
+			lines[k][0][i].R0.
+				MulByElement(
+					&lines[k][0][i].R0,
+					&xNegOverY[k],
+				)
 			lines[k][0][i].R1.
 				MulByElement(
 					&lines[k][0][i].R1,
 					&yInv[k],
 				)
-			lines[k][0][i].R0.
-				MulByElement(&lines[k][0][i].R0,
-					&xNegOverY[k],
-				)
+
 			if LoopCounter[i] == 0 {
 				// ℓ × res
-				result.MulBy01(
-					&lines[k][0][i].R1,
+				result.MulBy34(
 					&lines[k][0][i].R0,
+					&lines[k][0][i].R1,
 				)
-
 			} else {
-				lines[k][1][i].R1.
-					MulByElement(
-						&lines[k][1][i].R1,
-						&yInv[k],
-					)
+				// line evaluation at P[k]
 				lines[k][1][i].R0.
 					MulByElement(
 						&lines[k][1][i].R0,
 						&xNegOverY[k],
 					)
-				prodLines = fptower.Mul01By01(
-					&lines[k][0][i].R1, &lines[k][0][i].R0,
-					&lines[k][1][i].R1, &lines[k][1][i].R0,
+				lines[k][1][i].R1.
+					MulByElement(
+						&lines[k][1][i].R1,
+						&yInv[k],
+					)
+				// ℓ × ℓ
+				prodLines = fptower.Mul34By34(
+					&lines[k][0][i].R0, &lines[k][0][i].R1,
+					&lines[k][1][i].R0, &lines[k][1][i].R1,
 				)
-				result.MulBy01245(&prodLines)
+				// (ℓ × ℓ) × res
+				result.MulBy01234(&prodLines)
 			}
 		}
 	}
 
-	// negative x₀
-	result.Conjugate(&result)
-
 	return result, nil
 }
 
