@@ -9,7 +9,7 @@ import (
 	"sync"
 
 	"github.com/consensys/gnark-crypto/ecc"
-	"github.com/consensys/gnark-crypto/ecc/bls24-315/fr"
+	"github.com/consensys/gnark-crypto/ecc/bls24-317/fr"
 )
 
 var bigIntPool = sync.Pool{
@@ -54,14 +54,14 @@ func (z *E24) SetOne() *E24 {
 	return z
 }
 
-// Add set z=x+y in E24 and return z
+// Add sets z=x+y in E24 and return z
 func (z *E24) Add(x, y *E24) *E24 {
 	z.D0.Add(&x.D0, &y.D0)
 	z.D1.Add(&x.D1, &y.D1)
 	return z
 }
 
-// Sub sets z to x sub y and return z
+// Sub sets z to x-y and return z
 func (z *E24) Sub(x, y *E24) *E24 {
 	z.D0.Sub(&x.D0, &y.D0)
 	z.D1.Sub(&x.D1, &y.D1)
@@ -105,7 +105,7 @@ func (z *E24) IsOne() bool {
 	return z.D0.IsOne() && z.D1.IsZero()
 }
 
-// Mul set z=x*y in E24 and return z
+// Mul sets z=x*y in E24 and return z
 func (z *E24) Mul(x, y *E24) *E24 {
 	var a, b, c E12
 	a.Add(&x.D0, &x.D1)
@@ -118,7 +118,7 @@ func (z *E24) Mul(x, y *E24) *E24 {
 	return z
 }
 
-// Square set z=x*x in E24 and return z
+// Square sets z=x*x in E24 and return z
 func (z *E24) Square(x *E24) *E24 {
 
 	//Algorithm 22 from https://eprint.iacr.org/2010/354.pdf
@@ -277,84 +277,6 @@ func (z *E24) DecompressKarabina(x *E24) *E24 {
 	return z
 }
 
-// BatchDecompressKarabina multiple Karabina's cyclotomic square results
-// if g3 != 0
-//
-//	g4 = (E * g5^2 + 3 * g1^2 - 2 * g2)/4g3
-//
-// if g3 == 0
-//
-//	g4 = 2g1g5/g2
-//
-// if g3=g2=0 then g4=g5=g1=0 and g0=1 (x=1)
-// Theorem 3.1 is well-defined for all x in Gϕₙ\{1}
-//
-// Divisions by 4g3 or g2 is batched using Montgomery batch inverse
-func BatchDecompressKarabina(x []E24) []E24 {
-
-	n := len(x)
-	if n == 0 {
-		return x
-	}
-
-	t0 := make([]E4, n)
-	t1 := make([]E4, n)
-	t2 := make([]E4, n)
-
-	var one E4
-	one.SetOne()
-
-	for i := 0; i < n; i++ {
-		// g3 == 0
-		if x[i].D1.C0.IsZero() {
-			t0[i].Mul(&x[i].D0.C1, &x[i].D1.C2).
-				Double(&t0[i])
-			// t1 = g2
-			t1[i].Set(&x[i].D0.C2)
-
-			// g3 != 0
-		} else {
-			// t0 = g1^2
-			t0[i].Square(&x[i].D0.C1)
-			// t1 = 3 * g1^2 - 2 * g2
-			t1[i].Sub(&t0[i], &x[i].D0.C2).
-				Double(&t1[i]).
-				Add(&t1[i], &t0[i])
-			// t0 = E * g5^2 + t1
-			t2[i].Square(&x[i].D1.C2)
-			t0[i].MulByNonResidue(&t2[i]).
-				Add(&t0[i], &t1[i])
-			// t1 = 4 * g3
-			t1[i].Double(&x[i].D1.C0).
-				Double(&t1[i])
-		}
-	}
-
-	t1 = BatchInvertE4(t1) // costs 1 inverse
-
-	for i := 0; i < n; i++ {
-		// z4 = g4
-		x[i].D1.C1.Mul(&t0[i], &t1[i])
-
-		// t1 = g2 * g1
-		t1[i].Mul(&x[i].D0.C2, &x[i].D0.C1)
-		// t2 = 2 * g4² - 3 * g2 * g1
-		t2[i].Square(&x[i].D1.C1)
-		t2[i].Sub(&t2[i], &t1[i])
-		t2[i].Double(&t2[i])
-		t2[i].Sub(&t2[i], &t1[i])
-
-		// t1 = g3 * g5 (g3s can be 0s)
-		t1[i].Mul(&x[i].D1.C0, &x[i].D1.C2)
-		// z0 = E * (2 * g4² + g3 * g5 - 3 * g2 * g1) + 1
-		t2[i].Add(&t2[i], &t1[i])
-		x[i].D0.C0.MulByNonResidue(&t2[i]).
-			Add(&x[i].D0.C0, &one)
-	}
-
-	return x
-}
-
 // Granger-Scott's cyclotomic square
 // https://eprint.iacr.org/2009/565.pdf, 3.2
 func (z *E24) CyclotomicSquare(x *E24) *E24 {
@@ -394,7 +316,7 @@ func (z *E24) CyclotomicSquare(x *E24) *E24 {
 	return z
 }
 
-// Inverse set z to the inverse of x in E24 and return z
+// Inverse sets z to the inverse of x in E24 and return z
 //
 // if x == 0, sets and returns z = x
 func (z *E24) Inverse(x *E24) *E24 {
@@ -412,8 +334,8 @@ func (z *E24) Inverse(x *E24) *E24 {
 	return z
 }
 
-// BatchInvertE24 returns a new slice with every element inverted.
-// Uses Montgomery batch inversion trick
+// BatchInvertE24 returns a new slice with every element in a inverted.
+// It uses Montgomery batch inversion trick.
 //
 // if a[i] == 0, returns result[i] = a[i]
 func BatchInvertE24(a []E24) []E24 {
@@ -623,12 +545,12 @@ func (z *E24) ExpGLV(x E24, k *big.Int) *E24 {
 	return z
 }
 
-// InverseUnitary inverse a unitary element
+// InverseUnitary inverses a unitary element
 func (z *E24) InverseUnitary(x *E24) *E24 {
 	return z.Conjugate(x)
 }
 
-// Conjugate set z to x conjugated and return z
+// Conjugate sets z to x conjugated and return z
 func (z *E24) Conjugate(x *E24) *E24 {
 	*z = *x
 	z.D1.Neg(&z.D1)
