@@ -3,13 +3,15 @@
 
 package fptower
 
-// MulBybTwistCurveCoeff multiplies by 1/(0,1)
+// MulBybTwistCurveCoeff multiplies by 4*(0,1)
 func (z *E4) MulBybTwistCurveCoeff(x *E4) *E4 {
 
 	var res E4
-	res.B0.Set(&x.B1)
-	res.B1.MulByNonResidueInv(&x.B0)
-	z.Set(&res)
+	res.B1.Set(&x.B0)
+	res.B0.MulByNonResidue(&x.B1)
+
+	z.Double(&res).
+		Double(z)
 
 	return z
 }
