@@ -9,8 +9,8 @@ import (
 	"sync"
 
 	"github.com/consensys/gnark-crypto/ecc"
-	"github.com/consensys/gnark-crypto/ecc/bw6-633/fp"
-	"github.com/consensys/gnark-crypto/ecc/bw6-633/fr"
+	"github.com/consensys/gnark-crypto/ecc/bw6-761/fp"
+	"github.com/consensys/gnark-crypto/ecc/bw6-761/fr"
 )
 
 var bigIntPool = sync.Pool{
@@ -43,8 +43,7 @@ func (z *E6) SetString(s0, s1, s2, s3, s4, s5 string) *E6 {
 
 // Set copies x into z and returns z
 func (z *E6) Set(x *E6) *E6 {
-	z.B0 = x.B0
-	z.B1 = x.B1
+	*z = *x
 	return z
 }
 
@@ -234,6 +233,8 @@ func (z *E6) DecompressKarabina(x *E6) *E6 {
 		// t1 = g2
 		t[1].Set(&x.B0.A2)
 
+		// g3 != 0
+
 		if t[1].IsZero() /* g2 == g3 == 0 */ {
 			return z.SetOne()
 		}
@@ -278,84 +279,6 @@ func (z *E6) DecompressKarabina(x *E6) *E6 {
 	return z
 }
 
-// BatchDecompressKarabina multiple Karabina's cyclotomic square results
-// if g3 != 0
-//
-//	g4 = (E * g5^2 + 3 * g1^2 - 2 * g2)/4g3
-//
-// if g3 == 0
-//
-//	g4 = 2g1g5/g2
-//
-// if g3=g2=0 then g4=g5=g1=0 and g0=1 (x=1)
-// Theorem 3.1 is well-defined for all x in Gϕₙ\{1}
-//
-// Divisions by 4g3 or g2 is batched using Montgomery batch inverse
-func BatchDecompressKarabina(x []E6) []E6 {
-
-	n := len(x)
-	if n == 0 {
-		return x
-	}
-
-	t0 := make([]fp.Element, n)
-	t1 := make([]fp.Element, n)
-	t2 := make([]fp.Element, n)
-
-	var one fp.Element
-	one.SetOne()
-
-	for i := 0; i < n; i++ {
-		// g3 == 0
-		if x[i].B1.A0.IsZero() {
-			t0[i].Mul(&x[i].B0.A1, &x[i].B1.A2).
-				Double(&t0[i])
-			// t1 = g2
-			t1[i].Set(&x[i].B0.A2)
-
-			// g3 != 0
-		} else {
-			// t0 = g1²
-			t0[i].Square(&x[i].B0.A1)
-			// t1 = 3 * g1² - 2 * g2
-			t1[i].Sub(&t0[i], &x[i].B0.A2).
-				Double(&t1[i]).
-				Add(&t1[i], &t0[i])
-			// t0 = E * g5² + t1
-			t2[i].Square(&x[i].B1.A2)
-			t0[i].MulByNonResidue(&t2[i]).
-				Add(&t0[i], &t1[i])
-			// t1 = 1/(4 * g3)
-			t1[i].Double(&x[i].B1.A0).
-				Double(&t1[i])
-		}
-	}
-
-	t1 = fp.BatchInvert(t1) // costs 1 inverse
-
-	for i := 0; i < n; i++ {
-		// z4 = g4
-		x[i].B1.A1.Mul(&t0[i], &t1[i])
-
-		// t1 = g2 * g1
-		t1[i].Mul(&x[i].B0.A2, &x[i].B0.A1)
-		// t2 = 2 * g4^2 - 3 * g2 * g1
-		t2[i].Square(&x[i].B1.A1)
-		t2[i].Sub(&t2[i], &t1[i])
-		t2[i].Double(&t2[i])
-		t2[i].Sub(&t2[i], &t1[i])
-
-		// t1 = g3 * g5 (g3s can be 0s)
-		t1[i].Mul(&x[i].B1.A0, &x[i].B1.A2)
-		// z0 = E * (2 * g4^2 + g3 * g5 - 3 * g2 * g1) + 1
-		t2[i].Add(&t2[i], &t1[i])
-		x[i].B0.A0.MulByNonResidue(&t2[i]).
-			Add(&x[i].B0.A0, &one)
-	}
-
-	return x
-}
-
 // Granger-Scott's cyclotomic square
 // https://eprint.iacr.org/2009/565.pdf, 3.2
 func (z *E6) CyclotomicSquare(x *E6) *E6 {
@@ -636,8 +559,7 @@ func (z *E6) Conjugate(x *E6) *E6 {
 }
 
 // SizeOfGT represents the size in bytes that a GT element need in binary form
-const SizeOfGT = sizeOfFp * 6
-const sizeOfFp = 80
+const SizeOfGT = fp.Bytes * 6
 
 // Bytes returns the regular (non montgomery) value
 // of z as a big-endian byte array.
@@ -645,30 +567,30 @@ const sizeOfFp = 80
 func (z *E6) Bytes() (r [SizeOfGT]byte) {
 
 	offset := 0
-	var buf [sizeOfFp]byte
+	var buf [fp.Bytes]byte
 
 	buf = z.B1.A2.Bytes()
-	copy(r[offset:offset+sizeOfFp], buf[:])
-	offset += sizeOfFp
+	copy(r[offset:offset+fp.Bytes], buf[:])
+	offset += fp.Bytes
 
 	buf = z.B1.A1.Bytes()
-	copy(r[offset:offset+sizeOfFp], buf[:])
-	offset += sizeOfFp
+	copy(r[offset:offset+fp.Bytes], buf[:])
+	offset += fp.Bytes
 
 	buf = z.B1.A0.Bytes()
-	copy(r[offset:offset+sizeOfFp], buf[:])
-	offset += sizeOfFp
+	copy(r[offset:offset+fp.Bytes], buf[:])
+	offset += fp.Bytes
 
 	buf = z.B0.A2.Bytes()
-	copy(r[offset:offset+sizeOfFp], buf[:])
-	offset += sizeOfFp
+	copy(r[offset:offset+fp.Bytes], buf[:])
+	offset += fp.Bytes
 
 	buf = z.B0.A1.Bytes()
-	copy(r[offset:offset+sizeOfFp], buf[:])
-	offset += sizeOfFp
+	copy(r[offset:offset+fp.Bytes], buf[:])
+	offset += fp.Bytes
 
 	buf = z.B0.A0.Bytes()
-	copy(r[offset:offset+sizeOfFp], buf[:])
+	copy(r[offset:offset+fp.Bytes], buf[:])
 
 	return
 }
@@ -681,17 +603,17 @@ func (z *E6) SetBytes(e []byte) error {
 		return errors.New("invalid buffer size")
 	}
 	offset := 0
-	z.B1.A2.SetBytes(e[offset : offset+sizeOfFp])
-	offset += sizeOfFp
-	z.B1.A1.SetBytes(e[offset : offset+sizeOfFp])
-	offset += sizeOfFp
-	z.B1.A0.SetBytes(e[offset : offset+sizeOfFp])
-	offset += sizeOfFp
-	z.B0.A2.SetBytes(e[offset : offset+sizeOfFp])
-	offset += sizeOfFp
-	z.B0.A1.SetBytes(e[offset : offset+sizeOfFp])
-	offset += sizeOfFp
-	z.B0.A0.SetBytes(e[offset : offset+sizeOfFp])
+	z.B1.A2.SetBytes(e[offset : offset+fp.Bytes])
+	offset += fp.Bytes
+	z.B1.A1.SetBytes(e[offset : offset+fp.Bytes])
+	offset += fp.Bytes
+	z.B1.A0.SetBytes(e[offset : offset+fp.Bytes])
+	offset += fp.Bytes
+	z.B0.A2.SetBytes(e[offset : offset+fp.Bytes])
+	offset += fp.Bytes
+	z.B0.A1.SetBytes(e[offset : offset+fp.Bytes])
+	offset += fp.Bytes
+	z.B0.A0.SetBytes(e[offset : offset+fp.Bytes])
 
 	return nil
 }
@@ -699,7 +621,7 @@ func (z *E6) SetBytes(e []byte) error {
 // IsInSubGroup ensures GT/E6 is in correct subgroup
 func (z *E6) IsInSubGroup() bool {
 	var tmp, a, _a, b E6
-	var t [13]E6
+	var t [6]E6
 
 	// check z^(phi_k(p)) == 1
 	a.Frobenius(z)
@@ -713,75 +635,47 @@ func (z *E6) IsInSubGroup() bool {
 	_a.Frobenius(z)
 	a.CyclotomicSquare(&_a).Mul(&a, &_a) // z^(3p)
 
-	// t(x)-1 = (-10-4x-13x²+6x³+7x⁴-23x⁵+19x⁶-12x⁷+2x⁸+11x⁹-7x¹⁰)/3
-	t[0].CyclotomicSquare(z)     // z²
-	t[1].CyclotomicSquare(&t[0]) // z⁴
+	// t(x)-1 = (13x⁶ − 23x⁵ − 9x⁴ + 35x³ + 10x + 19)/3
+	t[0].CyclotomicSquare(z) // z²
+	t[1].CyclotomicSquare(&t[0]).
+		CyclotomicSquare(&t[1]) // z⁸
 	t[2].CyclotomicSquare(&t[1]).
 		Mul(&t[2], &t[0]).
-		Conjugate(&t[2]) // *z^(-10)
-	t[3].Expt(&t[1]).
-		Conjugate(&t[3]) // *z^(-4u)
-	t[4].Conjugate(&t[1]).
-		Mul(&t[4], &t[2]).
-		Mul(&t[4], z).
+		Mul(&t[2], z) // z¹⁹*
+	t[3].Mul(&t[0], &t[1]).
+		Expt(&t[3]) // z^(10u)*
+	t[4].CyclotomicSquare(&t[3]).
+		Mul(&t[4], &t[3]) // z^(30u)
+	t[0].CyclotomicSquare(&t[0]).
+		Mul(&t[0], z).
+		Expt(&t[0]) // z^(5u)
+	t[4].Mul(&t[4], &t[0]).
 		Expt(&t[4]).
-		Expt(&t[4]) // *z^(-13u²)
-	t[5].Mul(&t[0], &t[1]).
-		Expt(&t[5]).
-		Expt(&t[5]).
-		Expt(&t[5]) // *z^(6u³)
-	tmp.Expt(z).
-		Expt(&tmp).
-		Expt(&tmp) // z^(u³)
-	t[6].Mul(&tmp, &t[5]).
-		Expt(&t[6]) // *z^(7u⁴)
-	t[7].CyclotomicSquare(&t[5]).
-		CyclotomicSquare(&t[7]) // z^(24u³)
-	tmp.Conjugate(&tmp) // z^(-u³)
-	t[7].Mul(&t[7], &tmp).
-		Conjugate(&t[7]).
-		Expt(&t[7]).
-		Expt(&t[7]) // *z^(-23u⁵)
-	t[8].Conjugate(&t[4]).
-		Expt(&t[8]).
-		Mul(&t[8], &t[5]).
-		Expt(&t[8]).
-		Expt(&t[8]).
-		Expt(&t[8]) // *z^(19u⁶)
-	t[9].Conjugate(&t[5]).
-		CyclotomicSquare(&t[9]).
-		Expt(&t[9]).
-		Expt(&t[9]).
-		Expt(&t[9]).
-		Expt(&t[9]) // *z^(-12u⁷)
-	tmp.Expt(&t[7]).
-		Expt(&tmp) // z^(-23u⁷)
-	t[10].Conjugate(&t[9]).
-		CyclotomicSquare(&t[10]).
-		Mul(&t[10], &tmp) // z^(u⁷)
-	t[11].Mul(&t[9], &t[10]).
-		Conjugate(&t[11]).
-		Expt(&t[11]).
-		Expt(&t[11]) // *z^(11u⁹)
-	t[10].Expt(&t[10]).
-		CyclotomicSquare(&t[10]) // *z^(2u⁸)
-	t[12].Conjugate(&t[10]).
-		CyclotomicSquare(&t[12]).
-		Expt(&t[12]).
-		Mul(&t[12], &t[11]).
-		Expt(&t[12]).
-		Conjugate(&t[12]) // *z^(-7u¹⁰)
+		Expt(&t[4]) // z^(35u³)*
+	t[1].Mul(&t[1], z).
+		Expt(&t[1]).
+		Expt(&t[1]).
+		Expt(&t[1]).
+		Expt(&t[1]).
+		Conjugate(&t[1]) // z^(-9u⁴)*
+	t[0].Expt(&t[0]).
+		Expt(&t[0]).
+		Expt(&t[0]).
+		Conjugate(&t[0]) // z^(-5u⁴)
+	t[5].CyclotomicSquare(&t[1]).
+		Mul(&t[5], &t[0]).
+		Expt(&t[5]) // z^(-23u⁵)*
+	tmp.CyclotomicSquare(&t[1]).
+		Conjugate(&tmp) // z^(18u⁴)
+	t[0].Mul(&t[0], &tmp).
+		Expt(&t[0]).
+		Expt(&t[0]) // z^(13u⁶)*
 
 	b.Mul(&t[2], &t[3]).
 		Mul(&b, &t[4]).
+		Mul(&b, &t[1]).
 		Mul(&b, &t[5]).
-		Mul(&b, &t[6]).
-		Mul(&b, &t[7]).
-		Mul(&b, &t[8]).
-		Mul(&b, &t[9]).
-		Mul(&b, &t[10]).
-		Mul(&b, &t[11]).
-		Mul(&b, &t[12]) // z^(3(t-1))
+		Mul(&b, &t[0]) // z^(3(t-1))
 
 	return a.Equal(&b)
 }
