@@ -1,13 +1,23 @@
-// Copyright 2020-2025 Consensys Software Inc.
-// Licensed under the Apache License, Version 2.0. See the LICENSE file for details.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 
-package bw6633
+package bw6761
 
 import (
 	"errors"
 
-	"github.com/consensys/gnark-crypto/ecc/bw6-633/fp"
-	"github.com/consensys/gnark-crypto/ecc/bw6-633/internal/fptower"
+	"github.com/consensys/gnark-crypto/ecc/bw6-761/fp"
+	"github.com/consensys/gnark-crypto/ecc/bw6-761/internal/fptower"
 )
 
 // GT target group of the pairing
@@ -48,7 +58,7 @@ func PairingCheck(P []G1Affine, Q []G2Affine) (bool, error) {
 // FinalExponentiation computes the exponentiation (∏ᵢ zᵢ)ᵈ
 // where d = (p^6-1)/r = (p^6-1)/Φ_6(p) ⋅ Φ_6(p)/r = (p^3-1)(p+1)(p^2 - p +1)/r
 // we use instead d=s ⋅ (p^3-1)(p+1)(p^2 - p +1)/r
-// where s is the cofactor (x^5-x^4-x) (El Housni and Guillevic)
+// where s is the cofactor (x_0+1) (El Housni and Guillevic)
 func FinalExponentiation(z *GT, _z ...*GT) GT {
 
 	var result GT
@@ -74,53 +84,49 @@ func FinalExponentiation(z *GT, _z ...*GT) GT {
 		return result
 	}
 
-	// Hard part (up to permutation)
-	// (x₀^5-x₀^4-x₀)(p²-p+1)/r
-	// Algorithm 4.5 from https://yelhousni.github.io/phd.pdf
-	var a, b, c, d, e, f, g, h, i, t, mp GT
-	mp.Frobenius(&result)
-	a.ExptMinus1Squared(&mp)
-	a.ExptSquarePlus1(&a)
-	a.Mul(&result, &a)
-	t.Conjugate(&mp)
-	b.ExptPlus1(&a).
-		Mul(&b, &t)
-	t.CyclotomicSquare(&a).
-		Mul(&t, &a)
-	a.Conjugate(&t)
+	// 2. Hard part (up to permutation)
+	// (x₀+1)(p²-p+1)/r
+	// Algorithm 4.4 from https://yelhousni.github.io/phd.pdf
+	var a, b, c, d, e, f, g, h, i, j, k, t GT
+	a.ExptMinus1Square(&result)
+	t.Frobenius(&result)
+	a.Mul(&a, &t)
+	b.ExptPlus1(&a)
+	t.Conjugate(&result)
+	b.Mul(&b, &t)
+	t.CyclotomicSquare(&a)
+	a.Mul(&a, &t)
 	c.ExptMinus1Div3(&b)
 	d.ExptMinus1(&c)
-	d.ExptSquarePlus1(&d)
-	e.ExptMinus1Squared(&d)
-	e.ExptSquarePlus1(&e)
+	e.ExptMinus1Square(&d)
 	e.Mul(&e, &d)
-	f.ExptPlus1(&e).
-		Mul(&f, &c).
-		Conjugate(&f).
-		Mul(&f, &d)
-	g.Mul(&f, &d).
-		Conjugate(&g)
-	h.ExptPlus1(&g).
-		Mul(&h, &c).
-		Mul(&h, &b)
-	// ht = −7, hy = −1
-	// c1 = (ht-hy)/2 = -3
-	i.Expc1(&f).
-		Mul(&i, &e)
-	// c2 = (ht^2+3*hy^2)/4 = 13
-	t.CyclotomicSquare(&i).
-		Mul(&t, &i).
-		Mul(&t, &b)
-	i.Expc2(&h).
-		Mul(&i, &t)
-	result.Mul(&a, &i)
+	d.Conjugate(&d)
+	f.Mul(&d, &b)
+	g.ExptPlus1(&e)
+	g.Mul(&g, &f)
+	h.Mul(&g, &c)
+	i.Mul(&g, &d)
+	i.ExptPlus1(&i)
+	t.Conjugate(&f)
+	i.Mul(&i, &t)
+	j.Expc1(&h)
+	j.Mul(&j, &e)
+	k.CyclotomicSquare(&j)
+	k.Mul(&k, &j)
+	k.Mul(&k, &b)
+	t.Expc2(&i)
+	k.Mul(&k, &t)
+	result.Mul(&a, &k)
 
 	return result
 }
 
 // MillerLoop computes the multi-Miller loop
-// computes the multi-Miller loop ∏ᵢ MillerLoop(Pᵢ, Qᵢ)
+// ∏ᵢ MillerLoop(Pᵢ, Qᵢ) =
+// ∏ᵢ { fᵢ_{x₀+1+λ(x₀³-x₀²-x₀),Qᵢ}(Pᵢ) }
+//
 // Alg.2 in https://eprint.iacr.org/2021/1359.pdf
+// Eq. (6') in https://hackmd.io/@gnark/BW6-761-changes
 func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 	// check input size match
 	n := len(P)
@@ -143,7 +149,7 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 	n = len(p)
 
 	// precomputations
-	qProj0 := make([]g2Proj, n)
+	qProj1 := make([]g2Proj, n)
 	q1 := make([]G2Affine, n)
 	q1Neg := make([]G2Affine, n)
 	q0Neg := make([]G2Affine, n)
@@ -151,8 +157,8 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 		q1[k].Y.Neg(&q0[k].Y)
 		q0Neg[k].X.Set(&q0[k].X)
 		q0Neg[k].Y.Set(&q1[k].Y)
-		q1[k].X.Mul(&q0[k].X, &thirdRootOneG2)
-		qProj0[k].FromAffine(&q0[k])
+		q1[k].X.Mul(&q0[k].X, &thirdRootOneG1)
+		qProj1[k].FromAffine(&q1[k])
 		q1Neg[k].Neg(&q1[k])
 	}
 
@@ -162,14 +168,16 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 	var l, l0 lineEvaluation
 	var prodLines [5]fp.Element
 
+	var j int8
+
 	if n >= 1 {
-		// i = 157, separately to avoid an E12 Square
+		// i = 188, separately to avoid an E12 Square
 		// (Square(res) = 1² = 1)
 		// j = 0
 		// k = 0, separately to avoid MulBy014 (res × ℓ)
 		// (assign line to res)
-		// qProj0[0] ← 2qProj0[0] and l0 the tangent ℓ passing 2qProj0[0]
-		qProj0[0].doubleStep(&l0)
+		// qProj1[0] ← 2qProj1[0] and l0 the tangent ℓ passing 2qProj1[0]
+		qProj1[0].doubleStep(&l0)
 		// line evaluation at Q[0] (assign)
 		result.B0.A0.Set(&l0.r0)
 		result.B0.A1.Mul(&l0.r1, &p[0].X)
@@ -178,8 +186,8 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 
 	// k = 1
 	if n >= 2 {
-		// qProj0[1] ← 2qProj0[1] and l0 the tangent ℓ passing 2qProj0[1]
-		qProj0[1].doubleStep(&l0)
+		// qProj1[1] ← 2qProj1[1] and l0 the tangent ℓ passing 2qProj1[1]
+		qProj1[1].doubleStep(&l0)
 		// line evaluation at Q[1]
 		l0.r1.Mul(&l0.r1, &p[1].X)
 		l0.r2.Mul(&l0.r2, &p[1].Y)
@@ -193,8 +201,8 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 
 	// k >= 2
 	for k := 2; k < n; k++ {
-		// qProj0[k] ← 2qProj0[k] and l0 the tangent ℓ passing 2qProj0[k]
-		qProj0[k].doubleStep(&l0)
+		// qProj1[k] ← 2qProj1[k] and l0 the tangent ℓ passing 2qProj1[k]
+		qProj1[k].doubleStep(&l0)
 		// line evaluation at Q[k]
 		l0.r1.Mul(&l0.r1, &p[k].X)
 		l0.r2.Mul(&l0.r2, &p[k].Y)
@@ -202,68 +210,43 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 		result.MulBy014(&l0.r0, &l0.r1, &l0.r2)
 	}
 
-	for i := len(LoopCounter) - 3; i >= 1; i-- {
-		// (∏ᵢfᵢ)²
-		// mutualize the square among n Miller loops
+	for i := 187; i >= 1; i-- {
 		result.Square(&result)
 
-		j := LoopCounter[i]*3 + LoopCounter1[i]
+		j = LoopCounter1[i]*3 + LoopCounter[i]
 
 		for k := 0; k < n; k++ {
-			// qProj0[1] ← 2pProj0[1] and l0 the tangent ℓ qassing 2pProj0[1]
-			qProj0[k].doubleStep(&l0)
-			// line evaluation at Q[k]
+			qProj1[k].doubleStep(&l0)
 			l0.r1.Mul(&l0.r1, &p[k].X)
 			l0.r2.Mul(&l0.r2, &p[k].Y)
 
 			switch j {
-			// cases -4, -2, 2, 4 do not occur given the static LoopCounters
+			// cases -4, -2, 2, 4 do not occur, given the static LoopCounters
 			case -3:
-				// qProj0[k] ← qProj0[k]-q1[k] and
-				// l the line ℓ qassing qProj0[k] and -q1[k]
-				qProj0[k].addMixedStep(&l, &q1Neg[k])
-				// line evaluation at Q[k]
+				qProj1[k].addMixedStep(&l, &q1Neg[k])
 				l.r1.Mul(&l.r1, &p[k].X)
 				l.r2.Mul(&l.r2, &p[k].Y)
-				// ℓ × ℓ
-				prodLines = fptower.Mul014By014(&l.r0, &l.r1, &l.r2, &l0.r0, &l0.r1, &l0.r2)
-				// (ℓ × ℓ) × res
+				prodLines = fptower.Mul014By014(&l0.r0, &l0.r1, &l0.r2, &l.r0, &l.r1, &l.r2)
 				result.MulBy01245(&prodLines)
 			case -1:
-				// qProj0[k] ← qProj0[k]-q0[k] and
-				// l the line ℓ qassing qProj0[k] and -q0[k]
-				qProj0[k].addMixedStep(&l, &q0Neg[k])
-				// line evaluation at Q[k]
+				qProj1[k].addMixedStep(&l, &q0Neg[k])
 				l.r1.Mul(&l.r1, &p[k].X)
 				l.r2.Mul(&l.r2, &p[k].Y)
-				// ℓ × ℓ
-				prodLines = fptower.Mul014By014(&l.r0, &l.r1, &l.r2, &l0.r0, &l0.r1, &l0.r2)
-				// (ℓ × ℓ) × res
+				prodLines = fptower.Mul014By014(&l0.r0, &l0.r1, &l0.r2, &l.r0, &l.r1, &l.r2)
 				result.MulBy01245(&prodLines)
 			case 0:
-				// ℓ × res
 				result.MulBy014(&l0.r0, &l0.r1, &l0.r2)
 			case 1:
-				// qProj0[k] ← qProj0[k]+q0[k] and
-				// l the line ℓ qassing qProj0[k] and q0[k]
-				qProj0[k].addMixedStep(&l, &q0[k])
-				// line evaluation at Q[k]
+				qProj1[k].addMixedStep(&l, &q0[k])
 				l.r1.Mul(&l.r1, &p[k].X)
 				l.r2.Mul(&l.r2, &p[k].Y)
-				// ℓ × ℓ
-				prodLines = fptower.Mul014By014(&l.r0, &l.r1, &l.r2, &l0.r0, &l0.r1, &l0.r2)
-				// (ℓ × ℓ) × res
+				prodLines = fptower.Mul014By014(&l0.r0, &l0.r1, &l0.r2, &l.r0, &l.r1, &l.r2)
 				result.MulBy01245(&prodLines)
 			case 3:
-				// qProj0[k] ← qProj0[k]+q1[k] and
-				// l the line ℓ qassing qProj0[k] and q1[k]
-				qProj0[k].addMixedStep(&l, &q1[k])
-				// line evaluation at Q[k]
+				qProj1[k].addMixedStep(&l, &q1[k])
 				l.r1.Mul(&l.r1, &p[k].X)
 				l.r2.Mul(&l.r2, &p[k].Y)
-				// (ℓ × ℓ) × res
-				prodLines = fptower.Mul014By014(&l.r0, &l.r1, &l.r2, &l0.r0, &l0.r1, &l0.r2)
-				// (ℓ × ℓ) × res
+				prodLines = fptower.Mul014By014(&l0.r0, &l0.r1, &l0.r2, &l.r0, &l.r1, &l.r2)
 				result.MulBy01245(&prodLines)
 			default:
 				return GT{}, errors.New("invalid LoopCounter")
@@ -271,30 +254,22 @@ func MillerLoop(P []G1Affine, Q []G2Affine) (GT, error) {
 		}
 	}
 
-	// i = 0, j = 1
+	// i = 0, separately to avoid a point addition
+	// j = -3
 	result.Square(&result)
 	for k := 0; k < n; k++ {
-		// qProj0[1] ← 2pProj0[1] and l0 the tangent ℓ qassing 2pProj0[1]
-		qProj0[k].doubleStep(&l0)
-		// line evaluation at Q[k]
+		qProj1[k].doubleStep(&l0)
 		l0.r1.Mul(&l0.r1, &p[k].X)
 		l0.r2.Mul(&l0.r2, &p[k].Y)
-		// qProj0[k] ← qProj0[k]+q0[k] and
-		// l the line ℓ qassing qProj0[k] and q0[k]
-		qProj0[k].lineCompute(&l, &q0[k])
-		// line evaluation at Q[k]
+		qProj1[k].lineCompute(&l, &q1Neg[k])
 		l.r1.Mul(&l.r1, &p[k].X)
 		l.r2.Mul(&l.r2, &p[k].Y)
-		// ℓ × ℓ
-		prodLines = fptower.Mul014By014(&l.r0, &l.r1, &l.r2, &l0.r0, &l0.r1, &l0.r2)
-		// (ℓ × ℓ) × res
+		prodLines = fptower.Mul014By014(&l0.r0, &l0.r1, &l0.r2, &l.r0, &l.r1, &l.r2)
 		result.MulBy01245(&prodLines)
 	}
 
-	// negative x₀
-	result.Conjugate(&result)
-
 	return result, nil
+
 }
 
 // doubleStep doubles a point in Homogenous projective coordinates, and evaluates the line in Miller loop
@@ -309,7 +284,7 @@ func (p *g2Proj) doubleStep(evaluations *lineEvaluation) {
 	C.Square(&p.z)
 	D.Double(&C).
 		Add(&D, &C)
-	E.Double(&D).Double(&E).Double(&E)
+	E.Double(&D).Double(&E)
 	F.Double(&E).
 		Add(&F, &E)
 	G.Add(&B, &F)
@@ -436,33 +411,32 @@ func PrecomputeLines(Q G2Affine) (PrecomputedLines [2][len(LoopCounter) - 1]Line
 	// precomputations
 	var accQ, imQ, imQneg, negQ G2Affine
 	imQ.Y.Neg(&Q.Y)
+	imQ.X.Mul(&Q.X, &thirdRootOneG1)
 	negQ.X.Set(&Q.X)
 	negQ.Y.Set(&imQ.Y)
-	imQ.X.Mul(&Q.X, &thirdRootOneG2)
-	accQ.Set(&Q)
-	imQneg.Neg(&imQ)
+	accQ.Set(&imQ)
+	imQneg.X.Set(&imQ.X)
+	imQneg.Y.Set(&Q.Y)
 
-	for i := len(LoopCounter) - 2; i >= 0; i-- {
+	for i := len(LoopCounter) - 2; i > 0; i-- {
 
-		accQ.doubleStep(&PrecomputedLines[0][i])
-
-		j := LoopCounter[i]*3 + LoopCounter1[i]
-		switch j {
-		// cases -4, -2, 2, 4 do not occur given the static LoopCounters
+		switch LoopCounter1[i]*3 + LoopCounter[i] {
+		// cases -4, -2, 2, 4 do not occur, given the static LoopCounters
 		case -3:
-			accQ.addStep(&PrecomputedLines[1][i], &imQneg)
+			accQ.doubleAndAddStep(&PrecomputedLines[0][i], &PrecomputedLines[1][i], &imQneg)
 		case -1:
-			accQ.addStep(&PrecomputedLines[1][i], &negQ)
+			accQ.doubleAndAddStep(&PrecomputedLines[0][i], &PrecomputedLines[1][i], &negQ)
 		case 0:
-			continue
+			accQ.doubleStep(&PrecomputedLines[0][i])
 		case 1:
-			accQ.addStep(&PrecomputedLines[1][i], &Q)
+			accQ.doubleAndAddStep(&PrecomputedLines[0][i], &PrecomputedLines[1][i], &Q)
 		case 3:
-			accQ.addStep(&PrecomputedLines[1][i], &imQ)
+			accQ.doubleAndAddStep(&PrecomputedLines[0][i], &PrecomputedLines[1][i], &imQ)
 		default:
 			return [2][len(LoopCounter) - 1]LineEvaluationAff{}
 		}
 	}
+	accQ.tangentCompute(&PrecomputedLines[0][0])
 
 	return PrecomputedLines
 }
@@ -511,7 +485,7 @@ func MillerLoopFixedQ(P []G1Affine, lines [][2][len(LoopCounter) - 1]LineEvaluat
 	for i := len(LoopCounter) - 2; i >= 0; i-- {
 		result.Square(&result)
 
-		j := LoopCounter[i]*3 + LoopCounter1[i]
+		j := LoopCounter1[i]*3 + LoopCounter[i]
 		for k := 0; k < n; k++ {
 			lines[k][0][i].R1.
 				Mul(
@@ -548,9 +522,6 @@ func MillerLoopFixedQ(P []G1Affine, lines [][2][len(LoopCounter) - 1]LineEvaluat
 		}
 	}
 
-	// negative x₀
-	result.Conjugate(&result)
-
 	return result, nil
 
 }
@@ -608,3 +579,162 @@ func (p *G2Affine) addStep(evaluations *LineEvaluationAff, a *G2Affine) {
 	p.X.Set(&xr)
 	p.Y.Set(&yr)
 }
+
+func (p *G2Affine) doubleAndAddStep(evaluations1, evaluations2 *LineEvaluationAff, a *G2Affine) {
+	var n, d, l1, x3, l2, x4, y4 fp.Element
+
+	// compute λ1 = (y2-y1)/(x2-x1)
+	n.Sub(&p.Y, &a.Y)
+	d.Sub(&p.X, &a.X)
+	l1.Div(&n, &d)
+
+	// compute x3 =λ1²-x1-x2
+	x3.Square(&l1)
+	x3.Sub(&x3, &p.X)
+	x3.Sub(&x3, &a.X)
+
+	// omit y3 computation
+
+	// compute line1
+	evaluations1.R0.Set(&l1)
+	evaluations1.R1.Mul(&l1, &p.X)
+	evaluations1.R1.Sub(&evaluations1.R1, &p.Y)
+
+	// compute λ2 = -λ1-2y1/(x3-x1)
+	n.Double(&p.Y)
+	d.Sub(&x3, &p.X)
+	l2.Div(&n, &d)
+	l2.Add(&l2, &l1)
+	l2.Neg(&l2)
+
+	// compute x4 = λ2²-x1-x3
+	x4.Square(&l2)
+	x4.Sub(&x4, &p.X)
+	x4.Sub(&x4, &x3)
+
+	// compute y4 = λ2(x1 - x4)-y1
+	y4.Sub(&p.X, &x4)
+	y4.Mul(&l2, &y4)
+	y4.Sub(&y4, &p.Y)
+
+	// compute line2
+	evaluations2.R0.Set(&l2)
+	evaluations2.R1.Mul(&l2, &p.X)
+	evaluations2.R1.Sub(&evaluations2.R1, &p.Y)
+
+	p.X.Set(&x4)
+	p.Y.Set(&y4)
+}
+
+func (p *G2Affine) tangentCompute(evaluations *LineEvaluationAff) {
+
+	var n, d, λ fp.Element
+	// λ = 3x²/2y
+	n.Square(&p.X)
+	λ.Double(&n).
+		Add(&λ, &n)
+	d.Double(&p.Y)
+	λ.Div(&λ, &d)
+
+	evaluations.R0.Set(&λ)
+	evaluations.R1.Mul(&λ, &p.X).
+		Sub(&evaluations.R1, &p.Y)
+}
+
+// ------------------------
+// direct-extension pairing
+// ------------------------
+
+// MillerLoopDirect computes the multi-Miller loop using the direct E6 extension
+// and returns a towered E6 element. This version corresponds to gnark circuit.
+// ∏ᵢ MillerLoop(Pᵢ, Qᵢ) =
+// ∏ᵢ { fᵢ_{x₀+1+λ(x₀³-x₀²-x₀),Qᵢ}(Pᵢ) }
+//
+// Alg.2 in https://eprint.iacr.org/2021/1359.pdf
+// Eq. (6') in https://hackmd.io/@gnark/BW6-761-changes
+func MillerLoopDirect(P []G1Affine, Q []G2Affine) (GT, error) {
+	// check input size match
+	n := len(P)
+	if n == 0 {
+		return GT{}, errors.New("invalid inputs sizes")
+	}
+
+	lines := make([][2][len(LoopCounter) - 1]LineEvaluationAff, 0, len(Q))
+	for _, qi := range Q {
+		lines = append(lines, PrecomputeLines(qi))
+	}
+
+	if n != len(Q) {
+		return GT{}, errors.New("invalid inputs sizes")
+	}
+
+	// no need to filter infinity points:
+	// 		1. if Pᵢ=(0,0) then -x/y=1/y=0 by gnark-crypto convention and so
+	// 		lines R0 and R1 are 0. It happens that result will stay, through
+	// 		the Miller loop, in 𝔽p⁶ because MulBy01(0,0,1),
+	// 		Mul01By01(0,0,1,0,0,1) and MulBy01245 set result.C0 to 0. At the
+	// 		end result will be in a proper subgroup of Fp¹² so it be reduced to
+	// 		1 in FinalExponentiation.
+	//
+	//      and/or
+	//
+	// 		2. if Qᵢ=(0,0) then PrecomputeLines(Qᵢ) will return lines R0 and R1
+	// 		that are 0 because of gnark-convention (*/0==0) in doubleStep and
+	// 		addStep. Similarly to Pᵢ=(0,0) it happens that result be 1
+	// 		after the FinalExponentiation.
+
+	// precomputations
+	yInv := make([]fp.Element, n)
+	xNegOverY := make([]fp.Element, n)
+	for k := 0; k < n; k++ {
+		yInv[k].Set(&P[k].Y)
+	}
+	yInv = fp.BatchInvert(yInv)
+	for k := 0; k < n; k++ {
+		xNegOverY[k].Mul(&P[k].X, &yInv[k]).
+			Neg(&xNegOverY[k])
+	}
+
+	// f_{a0+λ*a1,Q}(P)
+	var result fptower.E6D
+	result.SetOne()
+
+	for i := len(LoopCounter) - 2; i >= 0; i-- {
+		result.Square(&result)
+
+		j := LoopCounter1[i]*3 + LoopCounter[i]
+		for k := 0; k < n; k++ {
+			lines[k][0][i].R1.
+				Mul(
+					&lines[k][0][i].R1,
+					&yInv[k],
+				)
+			lines[k][0][i].R0.
+				Mul(&lines[k][0][i].R0,
+					&xNegOverY[k],
+				)
+			result.MulBy023(
+				&lines[k][0][i].R1,
+				&lines[k][0][i].R0,
+			)
+			if i > 0 && j != 0 {
+				lines[k][1][i].R1.
+					Mul(
+						&lines[k][1][i].R1,
+						&yInv[k],
+					)
+				lines[k][1][i].R0.
+					Mul(
+						&lines[k][1][i].R0,
+						&xNegOverY[k],
+					)
+				result.MulBy023(
+					&lines[k][1][i].R1,
+					&lines[k][1][i].R0,
+				)
+			}
+		}
+	}
+
+	return *fptower.ToTower(&result), nil
+}
