@@ -50,7 +50,7 @@ type FFAmd64 struct {
 	mDefines             map[string]defineFn
 }
 
-type defineFn func(args ...amd64.Register)
+type defineFn func(args ...any)
 
 func (f *FFAmd64) StackSize(maxNbRegistersNeeded, nbRegistersReserved, minStackSize int) int {
 	got := amd64.NbRegisters - nbRegistersReserved
@@ -63,6 +63,7 @@ func (f *FFAmd64) StackSize(maxNbRegistersNeeded, nbRegistersReserved, minStackS
 }
 
 func (f *FFAmd64) DefineFn(name string) (fn defineFn, err error) {
+	name = strings.ToUpper(name)
 	fn, ok := f.mDefines[name]
 	if !ok {
 		return nil, fmt.Errorf("function %s not defined", name)
@@ -70,7 +71,16 @@ func (f *FFAmd64) DefineFn(name string) (fn defineFn, err error) {
 	return fn, nil
 }
 
-func (f *FFAmd64) Define(name string, nbInputs int, fn defineFn) defineFn {
+func (f *FFAmd64) CallDefine(name string, args ...any) {
+	name = strings.ToUpper(name)
+	fn, ok := f.mDefines[name]
+	if !ok {
+		panic(fmt.Sprintf("function %s not defined", name))
+	}
+	fn(args...)
+}
+
+func (f *FFAmd64) Define(name string, nbInputs int, fn defineFn, forceGet ...bool) defineFn {
 
 	inputs := make([]string, nbInputs)
 	for i := 0; i < nbInputs; i++ {
@@ -78,25 +88,31 @@ func (f *FFAmd64) Define(name string, nbInputs int, fn defineFn) defineFn {
 	}
 	name = strings.ToUpper(name)
 
-	for _, ok := f.mDefines[name]; ok; {
+	for fn, ok := f.mDefines[name]; ok; {
+		if len(forceGet) > 0 && forceGet[0] {
+			// in that case, we don't redefine the define;
+			// user explicitly asked for it
+			return fn
+		}
+		panic("WARNING: function name already defined")
 		// name already exist, for code generation purpose we add a suffix
 		// should happen only with e2 deprecated functions
 		// fmt.Println("WARNING: function name already defined, adding suffix")
-		i := 0
-		for {
-			newName := fmt.Sprintf("%s_%d", name, i)
-			if _, ok := f.mDefines[newName]; !ok {
-				name = newName
-				goto startDefine
-			}
-			i++
-		}
+		// i := 0
+		// for {
+		// 	newName := fmt.Sprintf("%s_%d", name, i)
+		// 	if _, ok := f.mDefines[newName]; !ok {
+		// 		name = newName
+		// 		goto startDefine
+		// 	}
+		// 	i++
+		// }
 	}
-startDefine:
+	// startDefine:
 
 	f.StartDefine()
 	f.WriteLn("#define " + name + "(" + strings.Join(inputs, ", ") + ")")
-	inputsRegisters := make([]amd64.Register, nbInputs)
+	inputsRegisters := make([]any, nbInputs)
 	for i := 0; i < nbInputs; i++ {
 		inputsRegisters[i] = amd64.Register(inputs[i])
 	}
@@ -104,13 +120,24 @@ startDefine:
 	f.EndDefine()
 	f.WriteLn("")
 
-	toReturn := func(args ...amd64.Register) {
+	toReturn := func(args ...any) {
 		if len(args) != nbInputs {
 			panic("invalid number of arguments")
 		}
 		inputsStr := make([]string, len(args))
 		for i := 0; i < len(args); i++ {
-			inputsStr[i] = string(args[i])
+			switch t := args[i].(type) {
+			case amd64.Register:
+				inputsStr[i] = string(t)
+			case amd64.VectorRegister:
+				inputsStr[i] = string(t)
+			case amd64.MaskRegister:
+				inputsStr[i] = string(t)
+			case string:
+				inputsStr[i] = t
+			default:
+				panic("invalid argument type")
+			}
 		}
 		f.WriteLn(name + "(" + strings.Join(inputsStr, ", ") + ")")
 	}
@@ -219,7 +246,7 @@ func GenerateCommonASM(w io.Writer, nbWords, nbBits int, hasVector bool) error {
 	f.WriteLn("")
 
 	if nbWords == 1 {
-		if nbBits == 31 {
+		if nbBits <= 31 {
 			return GenerateF31ASM(f, hasVector)
 		} else {
 			panic("not implemented")
@@ -261,6 +288,112 @@ func GenerateCommonASM(w io.Writer, nbWords, nbBits int, hasVector bool) error {
 	return nil
 }
 
+func GenerateF31FFTKernels(w io.Writer, nbBits int, kernels []int) error {
+	if nbBits != 31 {
+		return fmt.Errorf("only 31 bits supported for now")
+	}
+	f := NewFFAmd64(w, 1)
+
+	f.WriteLn("")
+	f.Comment("Code generated by gnark-crypto/generator. DO NOT EDIT.")
+	f.Comment("Refer to the generator for more documentation.")
+	f.Comment("Some sub-functions are derived from Plonky3:")
+	f.Comment("https://github.com/Plonky3/Plonky3/blob/36e619f3c6526ee86e2e5639a24b3224e1c1700f/monty-31/src/x86_64_avx512/packing.rs#L319")
+	f.WriteLn("")
+	f.WriteLn("#include \"textflag.h\"")
+	f.WriteLn("#include \"funcdata.h\"")
+	f.WriteLn("#include \"go_asm.h\"")
+	f.WriteLn("")
+
+	f.generateFFTDefinesF31()
+	f.generateFFTInnerDITF31()
+	f.generateFFTInnerDIFF31()
+
+	// unroll kernels for size 256 for now.
+	fftKernels := []int{8}
+	for _, ksize := range fftKernels {
+		f.generateFFTKernelF31(ksize, true)
+		f.generateFFTKernelF31(ksize, false)
+	}
+
+	return nil
+}
+
+func GenerateF31E4(w io.Writer) error {
+	f := NewFFAmd64(w, 1)
+
+	f.WriteLn("")
+	f.Comment("Code generated by gnark-crypto/generator. DO NOT EDIT.")
+	f.Comment("Refer to the generator for more documentation.")
+	f.WriteLn("")
+	f.WriteLn("#include \"textflag.h\"")
+	f.WriteLn("#include \"funcdata.h\"")
+	f.WriteLn("#include \"go_asm.h\"")
+	f.WriteLn("")
+
+	f.generateMulAccE4()
+
+	return nil
+
+}
+
+func GenerateF31SIS(w io.Writer, nbBits int) error {
+	if nbBits != 31 {
+		return fmt.Errorf("only 31 bits supported for now")
+	}
+	f := NewFFAmd64(w, 1)
+
+	f.WriteLn("")
+	f.Comment("Code generated by gnark-crypto/generator. DO NOT EDIT.")
+	f.Comment("Refer to the generator for more documentation.")
+	f.Comment("Some sub-functions are derived from Plonky3:")
+	f.Comment("https://github.com/Plonky3/Plonky3/blob/36e619f3c6526ee86e2e5639a24b3224e1c1700f/monty-31/src/x86_64_avx512/packing.rs#L319")
+	f.WriteLn("")
+	f.WriteLn("#include \"textflag.h\"")
+	f.WriteLn("#include \"funcdata.h\"")
+	f.WriteLn("#include \"go_asm.h\"")
+	f.WriteLn("")
+
+	f.generateFFTDefinesF31()
+
+	f.generateSIS512_16F31()
+	f.generateSISShuffleF31()
+	f.generateSISUnhuffleF31()
+
+	return nil
+}
+
+type Poseidon2Parameters struct {
+	Width, FullRounds, PartialRounds, SBoxDegree int
+	DiagInternal                                 []uint64
+}
+
+func GenerateF31Poseidon2(w io.Writer, nbBits int, params []Poseidon2Parameters) error {
+	if nbBits != 31 {
+		return fmt.Errorf("only 31 bits supported for now")
+	}
+	f := NewFFAmd64(w, 1)
+
+	f.WriteLn("")
+	f.Comment("Code generated by gnark-crypto/generator. DO NOT EDIT.")
+	f.Comment("Refer to the generator for more documentation.")
+	f.WriteLn("")
+	f.WriteLn("#include \"textflag.h\"")
+	f.WriteLn("#include \"funcdata.h\"")
+	f.WriteLn("#include \"go_asm.h\"")
+	f.WriteLn("")
+
+	for _, p := range params {
+		f.generatePoseidon2_F31(p)
+
+		if p.Width == 24 {
+			f.generatePoseidon2_F31_16x24(p)
+		}
+	}
+
+	return nil
+}
+
 func GenerateF31ASM(f *FFAmd64, hasVector bool) error {
 	if !hasVector {
 		return nil // nothing for now.
@@ -269,6 +402,8 @@ func GenerateF31ASM(f *FFAmd64, hasVector bool) error {
 	f.generateAddVecF31()
 	f.generateSubVecF31()
 	f.generateSumVecF31()
+	f.generateSumVecSmallF31(16)
+	f.generateSumVecSmallF31(24)
 	f.generateMulVecF31()
 	f.generateScalarMulVecF31()
 	f.generateInnerProdVecF31()
