@@ -63,16 +63,16 @@ func (f *FFAmd64) MulADX(registers *amd64.Registers, x, y func(int) string, t []
 		}
 	}
 
-	mac := f.Define("MACC", 3, func(args ...amd64.Register) {
+	mac := f.Define("MACC", 3, func(args ...any) {
 		in0 := args[0]
 		in1 := args[1]
 		in2 := args[2]
 		f.ADCXQ(in0, in1)
 		f.MULXQ(in2, amd64.AX, in0)
 		f.ADOXQ(amd64.AX, in1)
-	})
+	}, true)
 
-	divShift := f.Define("DIV_SHIFT", 0, func(_ ...amd64.Register) {
+	divShift := f.Define("DIV_SHIFT", 0, func(_ ...any) {
 		if !hasFreeRegister {
 			f.PUSHQ(A)
 		}
@@ -104,9 +104,9 @@ func (f *FFAmd64) MulADX(registers *amd64.Registers, x, y func(int) string, t []
 		f.ADCXQ(amd64.AX, t[f.NbWordsLastIndex])
 		f.ADOXQ(A, t[f.NbWordsLastIndex])
 
-	})
+	}, true)
 
-	mulWord0 := f.Define("MUL_WORD_0", 0, func(_ ...amd64.Register) {
+	mulWord0 := f.Define("MUL_WORD_0", 0, func(_ ...any) {
 		f.XORQ(amd64.AX, amd64.AX)
 		// for j=0 to N-1
 		//    (A,t[j])  := t[j] + x[j]*y[i] + A
@@ -127,9 +127,9 @@ func (f *FFAmd64) MulADX(registers *amd64.Registers, x, y func(int) string, t []
 		f.MOVQ(0, amd64.AX)
 		f.ADOXQ(amd64.AX, A)
 		divShift()
-	})
+	}, true)
 
-	mulWordN := f.Define("MUL_WORD_N", 0, func(args ...amd64.Register) {
+	mulWordN := f.Define("MUL_WORD_N", 0, func(args ...any) {
 		f.XORQ(amd64.AX, amd64.AX)
 		// for j=0 to N-1
 		//    (A,t[j])  := t[j] + x[j]*y[i] + A
@@ -142,7 +142,7 @@ func (f *FFAmd64) MulADX(registers *amd64.Registers, x, y func(int) string, t []
 		f.ADCXQ(amd64.AX, A)
 		f.ADOXQ(amd64.AX, A)
 		divShift()
-	})
+	}, true)
 
 	f.Comment("mul body")
 
