@@ -7,9 +7,19 @@ import (
 type Option func(*generatorConfig)
 
 type generatorConfig struct {
-	fftConfig *config.FFT
-	asmConfig *config.Assembly
-	withSIS   bool
+	fftConfig      *config.FFT
+	asmConfig      *config.Assembly
+	withSIS        bool
+	withPoseidon2  bool
+	withExtensions bool
+}
+
+func (cfg *generatorConfig) HasExtensions() bool {
+	return cfg.withExtensions
+}
+
+func (cfg *generatorConfig) HasPoseidon2() bool {
+	return cfg.withPoseidon2
 }
 
 func (cfg *generatorConfig) HasSIS() bool {
@@ -34,6 +44,18 @@ func WithSIS() Option {
 	}
 }
 
+func WithPoseidon2() Option {
+	return func(opt *generatorConfig) {
+		opt.withPoseidon2 = true
+	}
+}
+
+func WithExtensions() Option {
+	return func(opt *generatorConfig) {
+		opt.withExtensions = true
+	}
+}
+
 func WithFFT(cfg *config.FFT) Option {
 	return func(opt *generatorConfig) {
 		opt.fftConfig = cfg
