@@ -1,7 +1,6 @@
 package ecc
 
 import (
-	"fmt"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -14,44 +13,17 @@ import (
 )
 
 func Generate(conf config.Curve, baseDir string, bgen *bavard.BatchGenerator) error {
-
 	packageName := strings.ReplaceAll(conf.Name, "-", "")
 
 	var entries []bavard.Entry
 
-	// hash To curve
-	genHashToCurve := func(point *config.Point, suite config.HashSuite) error {
-		if suite == nil { //Nothing to generate. Bypass
-			return nil
-		}
-
-		entries = []bavard.Entry{
-			{File: filepath.Join(baseDir, fmt.Sprintf("hash_to_%s.go", point.PointName)), Templates: []string{"hash_to_curve.go.tmpl", "sswu.go.tmpl", "svdw.go.tmpl"}},
-			{File: filepath.Join(baseDir, fmt.Sprintf("hash_to_%s_test.go", point.PointName)), Templates: []string{"tests/hash_to_curve.go.tmpl"}}}
-
-		hashConf := suite.GetInfo(conf.Fp, point, conf.Name)
-
-		funcs := make(template.FuncMap)
-		funcs["asElement"] = hashConf.Field.Base.WriteElement
-		bavardOpts := []func(*bavard.Bavard) error{bavard.Funcs(funcs)}
-
-		return bgen.GenerateWithOptions(hashConf, packageName, "./ecc/template", bavardOpts, entries...)
-	}
-
-	if err := genHashToCurve(&conf.G1, conf.HashE1); err != nil {
-		return err
-	}
-	if err := genHashToCurve(&conf.G2, conf.HashE2); err != nil {
-		return err
-	}
-
 	// G1
 	entries = []bavard.Entry{
 		{File: filepath.Join(baseDir, "g1.go"), Templates: []string{"point.go.tmpl"}},
 		{File: filepath.Join(baseDir, "g1_test.go"), Templates: []string{"tests/point.go.tmpl"}},
 	}
 	// if not secp256k1, generate the lagrange transform
-	if conf.Name != config.SECP256K1.Name {
+	if conf.Name != config.SECP256K1.Name || conf.Name != config.GRUMPKIN.Name {
 		os.Remove(filepath.Join(baseDir, "g1_lagrange.go"))
 		os.Remove(filepath.Join(baseDir, "g1_lagrange_test.go"))
 	}
@@ -171,8 +143,8 @@ func Generate(conf config.Curve, baseDir string, bgen *bavard.BatchGenerator) er
 		return err
 	}
 
-	// No G2 for secp256k1
-	if conf.Equal(config.SECP256K1) {
+	// No G2 for secp256k1 and grumpkin
+	if conf.Equal(config.SECP256K1) || conf.Equal(config.GRUMPKIN) {
 		return nil
 	}
 
