@@ -2,7 +2,6 @@ package main
 
 import (
 	"fmt"
-	"github.com/consensys/gnark-crypto/internal/generator/mpcsetup"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -20,18 +19,17 @@ import (
 	"github.com/consensys/gnark-crypto/internal/generator/edwards/eddsa"
 	"github.com/consensys/gnark-crypto/internal/generator/fflonk"
 	fri "github.com/consensys/gnark-crypto/internal/generator/fri/template"
-	"github.com/consensys/gnark-crypto/internal/generator/gkr"
+	"github.com/consensys/gnark-crypto/internal/generator/hash_to_curve"
 	"github.com/consensys/gnark-crypto/internal/generator/hash_to_field"
 	"github.com/consensys/gnark-crypto/internal/generator/iop"
 	"github.com/consensys/gnark-crypto/internal/generator/kzg"
+	"github.com/consensys/gnark-crypto/internal/generator/mpcsetup"
 	"github.com/consensys/gnark-crypto/internal/generator/pairing"
 	"github.com/consensys/gnark-crypto/internal/generator/pedersen"
 	"github.com/consensys/gnark-crypto/internal/generator/permutation"
 	"github.com/consensys/gnark-crypto/internal/generator/plookup"
 	"github.com/consensys/gnark-crypto/internal/generator/polynomial"
 	"github.com/consensys/gnark-crypto/internal/generator/shplonk"
-	"github.com/consensys/gnark-crypto/internal/generator/sumcheck"
-	"github.com/consensys/gnark-crypto/internal/generator/test_vector_utils"
 	"github.com/consensys/gnark-crypto/internal/generator/tower"
 )
 
@@ -75,8 +73,14 @@ func main() {
 
 			conf.FpUnusedBits = 64 - (conf.Fp.NbBits % 64)
 
+			frInfo := fieldConfig.FieldDependency{
+				FieldPackagePath: "github.com/consensys/gnark-crypto/ecc/" + conf.Name + "/fr",
+				FieldPackageName: "fr",
+				ElementType:      "fr.Element",
+			}
+
 			frOpts := []generator.Option{generator.WithASM(asmConfig)}
-			if !(conf.Equal(config.STARK_CURVE) || conf.Equal(config.SECP256K1)) {
+			if !(conf.Equal(config.STARK_CURVE) || conf.Equal(config.SECP256K1) || conf.Equal(config.GRUMPKIN)) {
 				frOpts = append(frOpts, generator.WithFFT(fftConfig))
 			}
 			if conf.Equal(config.BLS12_377) {
@@ -99,6 +103,35 @@ func main() {
 				return
 			}
 
+			// generate mimc on fr
+			assertNoError(mimc.Generate(conf, filepath.Join(curveDir, "fr", "mimc"), bgen))
+
+			// generate poseidon2 on fr
+			assertNoError(poseidon2.Generate(conf, filepath.Join(curveDir, "fr", "poseidon2"), bgen))
+
+			// generate polynomial on fr
+			assertNoError(polynomial.Generate(frInfo, filepath.Join(curveDir, "fr", "polynomial"), true, bgen))
+
+			fpInfo := fieldConfig.FieldDependency{
+				FieldPackagePath: "github.com/consensys/gnark-crypto/ecc/" + conf.Name + "/fp",
+				FieldPackageName: "fp",
+				ElementType:      "fp.Element",
+			}
+
+			// generate wrapped hash-to-field for both fr and fp
+			assertNoError(hash_to_field.Generate(frInfo, filepath.Join(curveDir, "fr", "hash_to_field"), bgen))
+			assertNoError(hash_to_field.Generate(fpInfo, filepath.Join(curveDir, "fp", "hash_to_field"), bgen))
+
+			// generate hash to curve for both G1 and G2
+			assertNoError(hash_to_curve.Generate(conf, curveDir, bgen))
+
+			if conf.Equal(config.GRUMPKIN) {
+				return
+			}
+
+			// generate pedersen on fr
+			assertNoError(pedersen.Generate(conf, filepath.Join(curveDir, "fr", "pedersen"), bgen))
+
 			// generate tower of extension
 			assertNoError(tower.Generate(conf, filepath.Join(curveDir, "internal", "fptower"), bgen))
 
@@ -120,62 +153,18 @@ func main() {
 			// generate fflonk on fr
 			assertNoError(fflonk.Generate(conf, filepath.Join(curveDir, "fflonk"), bgen))
 
-			// generate pedersen on fr
-			assertNoError(pedersen.Generate(conf, filepath.Join(curveDir, "fr", "pedersen"), bgen))
-
 			// generate plookup on fr
 			assertNoError(plookup.Generate(conf, filepath.Join(curveDir, "fr", "plookup"), bgen))
 
 			// generate permutation on fr
 			assertNoError(permutation.Generate(conf, filepath.Join(curveDir, "fr", "permutation"), bgen))
 
-			// generate mimc on fr
-			assertNoError(mimc.Generate(conf, filepath.Join(curveDir, "fr", "mimc"), bgen))
-
-			// generate poseidon2 on fr
-			assertNoError(poseidon2.Generate(conf, filepath.Join(curveDir, "fr", "poseidon2"), bgen))
-
-			frInfo := config.FieldDependency{
-				FieldPackagePath: "github.com/consensys/gnark-crypto/ecc/" + conf.Name + "/fr",
-				FieldPackageName: "fr",
-				ElementType:      "fr.Element",
-			}
-
-			// generate polynomial on fr
-			assertNoError(polynomial.Generate(frInfo, filepath.Join(curveDir, "fr", "polynomial"), true, bgen))
-
 			// generate eddsa on companion curves
 			assertNoError(fri.Generate(conf, filepath.Join(curveDir, "fr", "fri"), bgen))
 
-			// generate sumcheck on fr
-			assertNoError(sumcheck.Generate(frInfo, filepath.Join(curveDir, "fr", "sumcheck"), bgen))
-
-			// generate gkr on fr
-			assertNoError(gkr.Generate(gkr.Config{
-				FieldDependency:         frInfo,
-				GenerateTests:           true,
-				TestVectorsRelativePath: "../../../../internal/generator/gkr/test_vectors",
-			}, filepath.Join(curveDir, "fr", "gkr"), bgen))
-
-			// generate test vector utils on fr
-			assertNoError(test_vector_utils.Generate(test_vector_utils.Config{
-				FieldDependency:             frInfo,
-				RandomizeMissingHashEntries: false,
-			}, filepath.Join(curveDir, "fr", "test_vector_utils"), bgen))
-
 			// generate iop functions
 			assertNoError(iop.Generate(conf, filepath.Join(curveDir, "fr", "iop"), bgen))
 
-			fpInfo := config.FieldDependency{
-				FieldPackagePath: "github.com/consensys/gnark-crypto/ecc/" + conf.Name + "/fp",
-				FieldPackageName: "fp",
-				ElementType:      "fp.Element",
-			}
-
-			// generate wrapped hash-to-field for both fr and fp
-			assertNoError(hash_to_field.Generate(frInfo, filepath.Join(curveDir, "fr", "hash_to_field"), bgen))
-			assertNoError(hash_to_field.Generate(fpInfo, filepath.Join(curveDir, "fp", "hash_to_field"), bgen))
-
 		}(conf)
 
 	}
@@ -196,13 +185,17 @@ func main() {
 
 	}
 
-	wg.Add(1)
-	go func() {
-		defer wg.Done()
-		assertNoError(test_vector_utils.GenerateRationals(bgen))
-	}()
 	wg.Wait()
 
+	// generate the ecc_field.go file
+	{
+		entries := []bavard.Entry{
+			{File: filepath.Join(baseDir, "ecc", "ecc_field.go"), Templates: []string{"ecc_field.go.tmpl"}},
+		}
+
+		assertNoError(bgen.Generate(config.Curves, "ecc", "./config/template", entries...))
+	}
+
 	// format the whole directory
 
 	cmd := exec.Command("gofmt", "-s", "-w", baseDir)
@@ -220,28 +213,6 @@ func main() {
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
 	assertNoError(cmd.Run())*/
-
-	wg.Add(2)
-
-	go func() {
-		// generate test vectors for sumcheck
-		cmd := exec.Command("go", "run", "./sumcheck/test_vectors")
-		cmd.Stdout = os.Stdout
-		cmd.Stderr = os.Stderr
-		assertNoError(cmd.Run())
-		wg.Done()
-	}()
-
-	go func() {
-		// generate test vectors for gkr
-		cmd := exec.Command("go", "run", "./gkr/test_vectors")
-		cmd.Stdout = os.Stdout
-		cmd.Stderr = os.Stderr
-		assertNoError(cmd.Run())
-		wg.Done()
-	}()
-
-	wg.Wait()
 }
 
 func assertNoError(err error) {
