@@ -1,4 +1,4 @@
-namespace Ckzg;
+namespace CkzgLib;
 
 public static partial class Ckzg
 {
@@ -160,7 +160,7 @@ public static partial class Ckzg
     {
         ThrowOnUninitializedTrustedSetup(ckzgSetup);
         ThrowOnInvalidLength(blob, nameof(blob), BytesPerBlob);
-        ThrowOnInvalidLength(commitment, nameof(proof), BytesPerCommitment);
+        ThrowOnInvalidLength(commitment, nameof(commitment), BytesPerCommitment);
         ThrowOnInvalidLength(proof, nameof(proof), BytesPerProof);
 
         fixed (byte* blobPtr = blob, commitmentPtr = commitment, proofPtr = proof)
@@ -345,4 +345,4 @@ public static partial class Ckzg
             throw new ArgumentException($"Invalid data size, got {data.Length}, expected {expectedLength}", fieldName);
     }
     #endregion
-}
\ No newline at end of file
+}
