@@ -8,8 +8,6 @@ mod serde;
 #[cfg(test)]
 mod test_formats;
 
-use arbitrary::Arbitrary;
-
 include!("./generated.rs");
 
 use alloc::boxed::Box;
@@ -88,7 +86,7 @@ impl fmt::Display for Error {
             | Self::InvalidKzgCommitment(s)
             | Self::InvalidTrustedSetup(s)
             | Self::MismatchLength(s) => f.write_str(s),
-            Self::LoadingTrustedSetupFailed(s) => write!(f, "KzgErrors: {:?}", s),
+            Self::LoadingTrustedSetupFailed(s) => write!(f, "KzgErrors: {s:?}"),
             Self::CError(s) => fmt::Debug::fmt(s, f),
         }
     }
@@ -100,6 +98,9 @@ impl From<KzgErrors> for Error {
     }
 }
 
+pub type CellsPerExtBlob = [Cell; CELLS_PER_EXT_BLOB];
+pub type ProofsPerExtBlob = [KZGProof; CELLS_PER_EXT_BLOB];
+
 #[derive(Debug)]
 pub enum KzgErrors {
     /// Failed to get current directory.
@@ -122,7 +123,7 @@ pub enum KzgErrors {
 pub fn hex_to_bytes(hex_str: &str) -> Result<Vec<u8>, Error> {
     let trimmed_str = hex_str.strip_prefix("0x").unwrap_or(hex_str);
     hex::decode(trimmed_str)
-        .map_err(|e| Error::InvalidHexFormat(format!("Failed to decode hex: {}", e)))
+        .map_err(|e| Error::InvalidHexFormat(format!("Failed to decode hex: {e}")))
 }
 
 /// Holds the parameters of a kzg trusted setup ceremony.
@@ -217,29 +218,26 @@ impl KZGSettings {
         // Load g1 Lagrange bytes
         g1_lagrange_bytes
             .chunks_mut(BYTES_PER_G1_POINT)
-            .map(|chunk| {
+            .try_for_each(|chunk| {
                 let line = lines.next().ok_or(KzgErrors::FileFormatError)?;
                 hex::decode_to_slice(line, chunk).map_err(|_| KzgErrors::ParseError)
-            })
-            .collect::<Result<(), KzgErrors>>()?;
+            })?;
 
         // Load g2 monomial bytes
         g2_monomial_bytes
             .chunks_mut(BYTES_PER_G2_POINT)
-            .map(|chunk| {
+            .try_for_each(|chunk| {
                 let line = lines.next().ok_or(KzgErrors::FileFormatError)?;
                 hex::decode_to_slice(line, chunk).map_err(|_| KzgErrors::ParseError)
-            })
-            .collect::<Result<(), KzgErrors>>()?;
+            })?;
 
         // Load g1 monomial bytes
         g1_monomial_bytes
             .chunks_mut(BYTES_PER_G1_POINT)
-            .map(|chunk| {
+            .try_for_each(|chunk| {
                 let line = lines.next().ok_or(KzgErrors::FileFormatError)?;
                 hex::decode_to_slice(line, chunk).map_err(|_| KzgErrors::ParseError)
-            })
-            .collect::<Result<(), KzgErrors>>()?;
+            })?;
 
         if lines.next().is_some() {
             return Err(KzgErrors::FileFormatError.into());
@@ -269,10 +267,9 @@ impl KZGSettings {
     ///
     /// Same as [`load_trusted_setup_file`](Self::load_trusted_setup_file)
     #[cfg_attr(not(feature = "std"), doc = ", but takes a `CStr` instead of a `Path`")]
-    /// .
     pub fn load_trusted_setup_file_inner(file_path: &CStr, precompute: u64) -> Result<Self, Error> {
         // SAFETY: `b"r\0"` is a valid null-terminated string.
-        const MODE: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"r\0") };
+        const MODE: &CStr = c"r";
 
         // SAFETY:
         // - .as_ptr(): pointer is not dangling because file_path has not been dropped.
@@ -291,7 +288,8 @@ impl KZGSettings {
             )));
         }
         let mut kzg_settings = MaybeUninit::<KZGSettings>::uninit();
-        let result = unsafe {
+
+        unsafe {
             let res = load_trusted_setup_file(kzg_settings.as_mut_ptr(), file_ptr, precompute);
             let _unchecked_close_result = libc::fclose(file_ptr);
 
@@ -302,9 +300,7 @@ impl KZGSettings {
                     "Invalid trusted setup: {res:?}"
                 )))
             }
-        };
-
-        result
+        }
     }
 
     pub fn blob_to_kzg_commitment(&self, blob: &Blob) -> Result<KZGCommitment, Error> {
@@ -444,12 +440,15 @@ impl KZGSettings {
         }
     }
 
-    pub fn compute_cells(&self, blob: &Blob) -> Result<Box<[Cell; CELLS_PER_EXT_BLOB]>, Error> {
-        let mut cells = [Cell::default(); CELLS_PER_EXT_BLOB];
+    pub fn compute_cells(&self, blob: &Blob) -> Result<Box<CellsPerExtBlob>, Error> {
+        let mut cells: Box<[Cell; CELLS_PER_EXT_BLOB]> = vec![Cell::default(); CELLS_PER_EXT_BLOB]
+            .into_boxed_slice()
+            .try_into()
+            .unwrap();
         unsafe {
             let res = compute_cells_and_kzg_proofs(cells.as_mut_ptr(), ptr::null_mut(), blob, self);
             if let C_KZG_RET::C_KZG_OK = res {
-                Ok(Box::new(cells))
+                Ok(cells)
             } else {
                 Err(Error::CError(res))
             }
@@ -459,20 +458,21 @@ impl KZGSettings {
     pub fn compute_cells_and_kzg_proofs(
         &self,
         blob: &Blob,
-    ) -> Result<
-        (
-            Box<[Cell; CELLS_PER_EXT_BLOB]>,
-            Box<[KZGProof; CELLS_PER_EXT_BLOB]>,
-        ),
-        Error,
-    > {
-        let mut cells = [Cell::default(); CELLS_PER_EXT_BLOB];
-        let mut proofs = [KZGProof::default(); CELLS_PER_EXT_BLOB];
+    ) -> Result<(Box<CellsPerExtBlob>, Box<ProofsPerExtBlob>), Error> {
+        let mut cells: Box<[Cell; CELLS_PER_EXT_BLOB]> = vec![Cell::default(); CELLS_PER_EXT_BLOB]
+            .into_boxed_slice()
+            .try_into()
+            .unwrap();
+        let mut proofs: Box<[KZGProof; CELLS_PER_EXT_BLOB]> =
+            vec![KZGProof::default(); CELLS_PER_EXT_BLOB]
+                .into_boxed_slice()
+                .try_into()
+                .unwrap();
         unsafe {
             let res =
                 compute_cells_and_kzg_proofs(cells.as_mut_ptr(), proofs.as_mut_ptr(), blob, self);
             if let C_KZG_RET::C_KZG_OK = res {
-                Ok((Box::new(cells), Box::new(proofs)))
+                Ok((cells, proofs))
             } else {
                 Err(Error::CError(res))
             }
@@ -483,13 +483,7 @@ impl KZGSettings {
         &self,
         cell_indices: &[u64],
         cells: &[Cell],
-    ) -> Result<
-        (
-            Box<[Cell; CELLS_PER_EXT_BLOB]>,
-            Box<[KZGProof; CELLS_PER_EXT_BLOB]>,
-        ),
-        Error,
-    > {
+    ) -> Result<(Box<CellsPerExtBlob>, Box<ProofsPerExtBlob>), Error> {
         if cell_indices.len() != cells.len() {
             return Err(Error::MismatchLength(format!(
                 "There are {} cell indices and {} cells",
@@ -497,8 +491,16 @@ impl KZGSettings {
                 cells.len()
             )));
         }
-        let mut recovered_cells = [Cell::default(); CELLS_PER_EXT_BLOB];
-        let mut recovered_proofs = [KZGProof::default(); CELLS_PER_EXT_BLOB];
+        let mut recovered_cells: Box<[Cell; CELLS_PER_EXT_BLOB]> =
+            vec![Cell::default(); CELLS_PER_EXT_BLOB]
+                .into_boxed_slice()
+                .try_into()
+                .unwrap();
+        let mut recovered_proofs: Box<[KZGProof; CELLS_PER_EXT_BLOB]> =
+            vec![KZGProof::default(); CELLS_PER_EXT_BLOB]
+                .into_boxed_slice()
+                .try_into()
+                .unwrap();
         unsafe {
             let res = recover_cells_and_kzg_proofs(
                 recovered_cells.as_mut_ptr(),
@@ -509,7 +511,7 @@ impl KZGSettings {
                 self,
             );
             if let C_KZG_RET::C_KZG_OK = res {
-                Ok((Box::new(recovered_cells), Box::new(recovered_proofs)))
+                Ok((recovered_cells, recovered_proofs))
             } else {
                 Err(Error::CError(res))
             }
@@ -684,7 +686,7 @@ impl KZGCommitment {
         if bytes.len() != BYTES_PER_COMMITMENT {
             return Err(Error::InvalidKzgCommitment(format!(
                 "Invalid byte length. Expected {} got {}",
-                BYTES_PER_PROOF,
+                BYTES_PER_COMMITMENT,
                 bytes.len(),
             )));
         }
@@ -824,6 +826,7 @@ impl Deref for KZGCommitment {
     }
 }
 
+#[allow(clippy::derivable_impls)]
 impl Default for Bytes32 {
     fn default() -> Self {
         Bytes32 { bytes: [0; 32] }
@@ -868,7 +871,8 @@ impl Default for Cell {
     }
 }
 
-impl Arbitrary<'_> for Bytes32 {
+#[cfg(feature = "arbitrary")]
+impl arbitrary::Arbitrary<'_> for Bytes32 {
     fn arbitrary(u: &mut arbitrary::Unstructured<'_>) -> arbitrary::Result<Self> {
         let mut bytes = [0u8; 32];
         u.fill_buffer(&mut bytes)?;
@@ -876,7 +880,8 @@ impl Arbitrary<'_> for Bytes32 {
     }
 }
 
-impl Arbitrary<'_> for Bytes48 {
+#[cfg(feature = "arbitrary")]
+impl arbitrary::Arbitrary<'_> for Bytes48 {
     fn arbitrary(u: &mut arbitrary::Unstructured<'_>) -> arbitrary::Result<Self> {
         let mut bytes = [0u8; 48];
         u.fill_buffer(&mut bytes)?;
@@ -884,7 +889,8 @@ impl Arbitrary<'_> for Bytes48 {
     }
 }
 
-impl Arbitrary<'_> for Blob {
+#[cfg(feature = "arbitrary")]
+impl arbitrary::Arbitrary<'_> for Blob {
     fn arbitrary(u: &mut arbitrary::Unstructured<'_>) -> arbitrary::Result<Self> {
         let mut bytes = [0u8; BYTES_PER_BLOB];
         u.fill_buffer(&mut bytes)?;
@@ -892,7 +898,8 @@ impl Arbitrary<'_> for Blob {
     }
 }
 
-impl Arbitrary<'_> for Cell {
+#[cfg(feature = "arbitrary")]
+impl arbitrary::Arbitrary<'_> for Cell {
     fn arbitrary(u: &mut arbitrary::Unstructured<'_>) -> arbitrary::Result<Self> {
         let mut bytes = [0u8; BYTES_PER_CELL];
         u.fill_buffer(&mut bytes)?;
@@ -1022,7 +1029,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_blob_to_kzg_commitment");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&blob.bytes).unwrap();
             }
@@ -1071,7 +1078,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_compute_kzg_proof");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&blob.bytes).unwrap();
                 file.write_all(&z.bytes).unwrap();
@@ -1117,7 +1124,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_compute_blob_kzg_proof");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&blob.bytes).unwrap();
                 file.write_all(&commitment.bytes).unwrap();
@@ -1164,7 +1171,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_verify_kzg_proof");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&commitment.bytes).unwrap();
                 file.write_all(&z.bytes).unwrap();
@@ -1212,7 +1219,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_verify_blob_kzg_proof");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&blob.bytes).unwrap();
                 file.write_all(&commitment.bytes).unwrap();
@@ -1259,7 +1266,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_verify_blob_kzg_proof_batch");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 for blob in &blobs {
                     file.write_all(&blob.bytes).unwrap();
@@ -1308,7 +1315,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_compute_cells");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&blob.bytes).unwrap();
             }
@@ -1350,7 +1357,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_compute_cells_and_kzg_proofs");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 file.write_all(&blob.bytes).unwrap();
             }
@@ -1400,7 +1407,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_recover_cells_and_kzg_proofs");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 for cell_index in &cell_indices {
                     file.write_all(&cell_index.to_le_bytes()).unwrap();
@@ -1457,7 +1464,7 @@ mod tests {
                     .join("corpus")
                     .join("fuzz_verify_cell_kzg_proof_batch");
                 fs::create_dir_all(&dir_path).unwrap();
-                let file_path = dir_path.join(format!("data_{}.bin", index));
+                let file_path = dir_path.join(format!("data_{index}.bin"));
                 let mut file = File::create(&file_path).unwrap();
                 for commitment in &commitments {
                     file.write_all(&commitment.bytes).unwrap();
