@@ -267,11 +267,9 @@ type ErrorResponse struct {
 	Message []byte `ssz-max:"256"`
 }
 
-type Dummy struct {
-}
+type Dummy struct{}
 
-type Interface interface {
-}
+type Interface interface{}
 
 type SyncCommittee struct {
 	PubKeys         [][]byte `json:"pubkeys" ssz-size:"512,48"`
@@ -303,18 +301,186 @@ type ExecutionPayload struct {
 }
 
 type ExecutionPayloadHeader struct {
-	ParentHash       []byte `protobuf:"bytes,1,opt,name=parent_hash,json=parentHash,proto3" json:"parent_hash" ssz-size:"32"`
-	FeeRecipient     []byte `protobuf:"bytes,2,opt,name=fee_recipient,json=feeRecipient,proto3" json:"fee_recipient" ssz-size:"20"`
-	StateRoot        []byte `protobuf:"bytes,3,opt,name=state_root,json=stateRoot,proto3" json:"state_root" ssz-size:"32"`
-	ReceiptsRoot     []byte `protobuf:"bytes,4,opt,name=receipts_root,json=receiptsRoot,proto3" json:"receipts_root" ssz-size:"32"`
-	LogsBloom        []byte `protobuf:"bytes,5,opt,name=logs_bloom,json=logsBloom,proto3" json:"logs_bloom" ssz-size:"256"`
-	PrevRandao       []byte `protobuf:"bytes,6,opt,name=prev_randao,json=prevRandao,proto3" json:"prev_randao" ssz-size:"32"`
-	BlockNumber      uint64 `protobuf:"varint,7,opt,name=block_number,json=blockNumber,proto3" json:"block_number"`
-	GasLimit         uint64 `protobuf:"varint,8,opt,name=gas_limit,json=gasLimit,proto3" json:"gas_limit"`
-	GasUsed          uint64 `protobuf:"varint,9,opt,name=gas_used,json=gasUsed,proto3" json:"gas_used"`
-	Timestamp        uint64 `protobuf:"varint,10,opt,name=timestamp,proto3" json:"timestamp"`
-	ExtraData        []byte `protobuf:"bytes,11,opt,name=extra_data,json=extraData,proto3" json:"extra_data" ssz-max:"32"`
-	BaseFeePerGas    []byte `protobuf:"bytes,12,opt,name=base_fee_per_gas,json=baseFeePerGas,proto3" json:"base_fee_per_gas" ssz-size:"32"`
-	BlockHash        []byte `protobuf:"bytes,13,opt,name=block_hash,json=blockHash,proto3" json:"block_hash" ssz-size:"32"`
-	TransactionsRoot []byte `protobuf:"bytes,14,opt,name=transactions_root,json=transactionsRoot,proto3" json:"transactions_root" ssz-size:"32"`
+	ParentHash       []byte `json:"parent_hash" ssz-size:"32"`
+	FeeRecipient     []byte `json:"fee_recipient" ssz-size:"20"`
+	StateRoot        []byte `json:"state_root" ssz-size:"32"`
+	ReceiptsRoot     []byte `json:"receipts_root" ssz-size:"32"`
+	LogsBloom        []byte `json:"logs_bloom" ssz-size:"256"`
+	PrevRandao       []byte `json:"prev_randao" ssz-size:"32"`
+	BlockNumber      uint64 `json:"block_number"`
+	GasLimit         uint64 `json:"gas_limit"`
+	GasUsed          uint64 `json:"gas_used"`
+	Timestamp        uint64 `json:"timestamp"`
+	ExtraData        []byte `json:"extra_data" ssz-max:"32"`
+	BaseFeePerGas    []byte `json:"base_fee_per_gas" ssz-size:"32"`
+	BlockHash        []byte `json:"block_hash" ssz-size:"32"`
+	TransactionsRoot []byte `json:"transactions_root" ssz-size:"32"`
+}
+
+// ExecutionPayloadTransactions provides information about transactions.
+type ExecutionPayloadTransactions struct {
+	Transactions [][]byte `ssz-max:"1048576,1073741824" ssz-size:"?,?"`
+}
+
+// Capella types
+
+type Uint256 [32]byte
+
+type ExecutionPayloadCapella struct {
+	ParentHash    [32]byte      `ssz-size:"32" json:"parent_hash"`
+	FeeRecipient  [20]byte      `ssz-size:"20" json:"fee_recipient"`
+	StateRoot     [32]byte      `ssz-size:"32" json:"state_root"`
+	ReceiptsRoot  [32]byte      `ssz-size:"32" json:"receipts_root"`
+	LogsBloom     [256]byte     `ssz-size:"256" json:"logs_bloom"`
+	PrevRandao    [32]byte      `ssz-size:"32" json:"prev_randao"`
+	BlockNumber   uint64        `json:"block_number"`
+	GasLimit      uint64        `json:"gas_limit"`
+	GasUsed       uint64        `json:"gas_used"`
+	Timestamp     uint64        `json:"timestamp"`
+	ExtraData     []byte        `ssz-max:"32" json:"extra_data"`
+	BaseFeePerGas Uint256       `ssz-size:"32" json:"base_fee_per_gas"`
+	BlockHash     [32]byte      `ssz-size:"32" json:"block_hash"`
+	Transactions  [][]byte      `ssz-max:"1048576,1073741824" ssz-size:"?,?" json:"transactions"`
+	Withdrawals   []*Withdrawal `json:"withdrawals" ssz-max:"16"`
+}
+
+type ExecutionPayloadHeaderCapella struct {
+	ParentHash       [32]byte  `json:"parent_hash" ssz-size:"32"`
+	FeeRecipient     [20]byte  `json:"fee_recipient" ssz-size:"20"`
+	StateRoot        [32]byte  `json:"state_root" ssz-size:"32"`
+	ReceiptsRoot     [32]byte  `json:"receipts_root" ssz-size:"32"`
+	LogsBloom        [256]byte `json:"logs_bloom" ssz-size:"256"`
+	PrevRandao       [32]byte  `json:"prev_randao" ssz-size:"32"`
+	BlockNumber      uint64    `json:"block_number"`
+	GasLimit         uint64    `json:"gas_limit"`
+	GasUsed          uint64    `json:"gas_used"`
+	Timestamp        uint64    `json:"timestamp"`
+	ExtraData        []byte    `json:"extra_data" ssz-max:"32"`
+	BaseFeePerGas    Uint256   `json:"base_fee_per_gas" ssz-size:"32"`
+	BlockHash        [32]byte  `json:"block_hash" ssz-size:"32"`
+	TransactionsRoot [32]byte  `json:"transactions_root" ssz-size:"32"`
+	WithdrawalRoot   [32]byte  `json:"withdrawals_root" ssz-size:"32"`
+}
+
+type BLSToExecutionChange struct {
+	ValidatorIndex     uint64   `json:"validator_index"`
+	FromBLSPubKey      [48]byte `json:"from_bls_pubkey" ssz-size:"48"`
+	ToExecutionAddress [20]byte `json:"to_execution_address" ssz-size:"20"`
+}
+
+type HistoricalSummary struct {
+	BlockSummaryRoot [32]byte `json:"block_summary_root" ssz-size:"32"`
+	StateSummaryRoot [32]byte `json:"state_summary_root" ssz-size:"32"`
+}
+
+type SignedBLSToExecutionChange struct {
+	Message   *BLSToExecutionChange `json:"message"`
+	Signature [96]byte              `json:"signature" ssz-size:"96"`
+}
+
+type Withdrawal struct {
+	Index          uint64   `json:"index"`
+	ValidatorIndex uint64   `json:"validator_index"`
+	Address        [20]byte `json:"address" ssz-size:"20"`
+	Amount         uint64   `json:"amount"`
+}
+
+type BeaconStateCapella struct {
+	GenesisTime                  uint64                         `json:"genesis_time"`
+	GenesisValidatorsRoot        [32]byte                       `json:"genesis_validators_root" ssz-size:"32"`
+	Slot                         uint64                         `json:"slot"`
+	Fork                         *Fork                          `json:"fork"`
+	LatestBlockHeader            *BeaconBlockHeader             `json:"latest_block_header"`
+	BlockRoots                   [8192][32]byte                 `json:"block_roots" ssz-size:"8192,32"`
+	StateRoots                   [8192][32]byte                 `json:"state_roots" ssz-size:"8192,32"`
+	HistoricalRoots              [][]byte                       `json:"historical_roots" ssz-max:"16777216" ssz-size:"?,32"`
+	Eth1Data                     *Eth1Data                      `json:"eth1_data"`
+	Eth1DataVotes                []*Eth1Data                    `json:"eth1_data_votes" ssz-max:"2048"`
+	Eth1DepositIndex             uint64                         `json:"eth1_deposit_index"`
+	Validators                   []*Validator                   `json:"validators" ssz-max:"1099511627776"`
+	Balances                     []uint64                       `json:"balances" ssz-max:"1099511627776"`
+	RandaoMixes                  [65536][32]byte                `json:"randao_mixes" ssz-size:"65536,32"`
+	Slashings                    []uint64                       `json:"slashings" ssz-size:"8192"`
+	PreviousEpochParticipation   []byte                         `json:"previous_epoch_participation" ssz-max:"1099511627776"`
+	CurrentEpochParticipation    []byte                         `json:"current_epoch_participation" ssz-max:"1099511627776"`
+	JustificationBits            [1]byte                        `json:"justification_bits" ssz-size:"1"`
+	PreviousJustifiedCheckpoint  *Checkpoint                    `json:"previous_justified_checkpoint"`
+	CurrentJustifiedCheckpoint   *Checkpoint                    `json:"current_justified_checkpoint"`
+	FinalizedCheckpoint          *Checkpoint                    `json:"finalized_checkpoint"`
+	InactivityScores             []uint64                       `json:"inactivity_scores" ssz-max:"1099511627776"`
+	CurrentSyncCommittee         *SyncCommittee                 `json:"current_sync_committee"`
+	NextSyncCommittee            *SyncCommittee                 `json:"next_sync_committee"`
+	LatestExecutionPayloadHeader *ExecutionPayloadHeaderCapella `json:"latest_execution_payload_header"`
+	NextWithdrawalIndex          uint64                         `json:"next_withdrawal_index"`
+	NextWithdrawalValidatorIndex uint64                         `json:"next_withdrawal_validator_index"`
+	HistoricalSummaries          []*HistoricalSummary           `json:"historical_summaries" ssz-max:"16777216"`
+}
+
+type SignedBeaconBlockCapella struct {
+	Block     *BeaconBlockCapella `json:"message"`
+	Signature []byte              `json:"signature" ssz-size:"96"`
+}
+
+type BeaconBlockCapella struct {
+	Slot          uint64                  `json:"slot"`
+	ProposerIndex uint64                  `json:"proposer_index"`
+	ParentRoot    [32]byte                `json:"parent_root" ssz-size:"32"`
+	StateRoot     [32]byte                `json:"state_root" ssz-size:"32"`
+	Body          *BeaconBlockBodyCapella `json:"body"`
+}
+
+type BeaconBlockBodyCapella struct {
+	RandaoReveal          []byte                        `json:"randao_reveal" ssz-size:"96"`
+	Eth1Data              *Eth1Data                     `json:"eth1_data"`
+	Graffiti              [32]byte                      `json:"graffiti" ssz-size:"32"`
+	ProposerSlashings     []*ProposerSlashing           `json:"proposer_slashings" ssz-max:"16"`
+	AttesterSlashings     []*AttesterSlashing           `json:"attester_slashings" ssz-max:"2"`
+	Attestations          []*Attestation                `json:"attestations" ssz-max:"128"`
+	Deposits              []*Deposit                    `json:"deposits" ssz-max:"16"`
+	VoluntaryExits        []*SignedVoluntaryExit        `json:"voluntary_exits" ssz-max:"16"`
+	SyncAggregate         *SyncAggregate                `json:"sync_aggregate"`
+	ExecutionPayload      *ExecutionPayloadCapella      `json:"execution_payload"`
+	BlsToExecutionChanges []*SignedBLSToExecutionChange `json:"bls_to_execution_changes" ssz-max:"16"`
+}
+
+// -- deneb --
+
+type ExecutionPayloadDeneb struct {
+	ParentHash    [32]byte      `ssz-size:"32" json:"parent_hash"`
+	FeeRecipient  [20]byte      `ssz-size:"20" json:"fee_recipient"`
+	StateRoot     [32]byte      `ssz-size:"32" json:"state_root"`
+	ReceiptsRoot  [32]byte      `ssz-size:"32" json:"receipts_root"`
+	LogsBloom     [256]byte     `ssz-size:"256" json:"logs_bloom"`
+	PrevRandao    [32]byte      `ssz-size:"32" json:"prev_randao"`
+	BlockNumber   uint64        `json:"block_number"`
+	GasLimit      uint64        `json:"gas_limit"`
+	GasUsed       uint64        `json:"gas_used"`
+	Timestamp     uint64        `json:"timestamp"`
+	ExtraData     []byte        `ssz-max:"32" json:"extra_data"`
+	BaseFeePerGas Uint256       `ssz-size:"32" json:"base_fee_per_gas"`
+	BlockHash     [32]byte      `ssz-size:"32" json:"block_hash"`
+	Transactions  [][]byte      `ssz-max:"1048576,1073741824" ssz-size:"?,?" json:"transactions"`
+	Withdrawals   []*Withdrawal `json:"withdrawals" ssz-max:"16"`
+	BlobGasUsed   uint64        `json:"blob_gas_used"`
+	ExcessBlobGas uint64        `json:"excess_blob_gas"`
+}
+
+type ExecutionPayloadHeaderDeneb struct {
+	ParentHash       [32]byte  `json:"parent_hash" ssz-size:"32"`
+	FeeRecipient     [20]byte  `json:"fee_recipient" ssz-size:"20"`
+	StateRoot        [32]byte  `json:"state_root" ssz-size:"32"`
+	ReceiptsRoot     [32]byte  `json:"receipts_root" ssz-size:"32"`
+	LogsBloom        [256]byte `json:"logs_bloom" ssz-size:"256"`
+	PrevRandao       [32]byte  `json:"prev_randao" ssz-size:"32"`
+	BlockNumber      uint64    `json:"block_number"`
+	GasLimit         uint64    `json:"gas_limit"`
+	GasUsed          uint64    `json:"gas_used"`
+	Timestamp        uint64    `json:"timestamp"`
+	ExtraData        []byte    `json:"extra_data" ssz-max:"32"`
+	BaseFeePerGas    Uint256   `json:"base_fee_per_gas" ssz-size:"32"`
+	BlockHash        [32]byte  `json:"block_hash" ssz-size:"32"`
+	TransactionsRoot [32]byte  `json:"transactions_root" ssz-size:"32"`
+	WithdrawalRoot   [32]byte  `json:"withdrawals_root" ssz-size:"32"`
+	BlobGasUsed      uint64    `json:"blob_gas_used"`
+	ExcessBlobGas    uint64    `json:"excess_blob_gas"`
 }
