@@ -10,6 +10,7 @@ import (
 	"go/parser"
 	"go/token"
 	"io/ioutil"
+	"log"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -29,7 +30,7 @@ const bytesPerLengthOffset = 4
 // using the Value object.
 // 3. Use the IR to print the encoding functions
 
-func Encode(source string, targets []string, output string, includePaths []string, excludeTypeNames map[string]bool) error {
+func Encode(source string, targets []string, output string, includePaths []string, excludeTypeNames map[string]bool, suffix string) error {
 	files, err := parseInput(source) // 1.
 	if err != nil {
 		return err
@@ -61,6 +62,7 @@ func Encode(source string, targets []string, output string, includePaths []strin
 		packName:         packName,
 		targets:          targets,
 		excludeTypeNames: excludeTypeNames,
+		suffix:           suffix,
 	}
 
 	if err := e.generateIR(); err != nil { // 2.
@@ -114,7 +116,7 @@ func parseInput(source string) (map[string]*ast.File, error) {
 	}
 	if ok {
 		// dir
-		astFiles, err := parser.ParseDir(token.NewFileSet(), source, nil, parser.AllErrors)
+		astFiles, err := parser.ParseDir(token.NewFileSet(), source, nil, parser.ParseComments)
 		if err != nil {
 			return nil, err
 		}
@@ -126,12 +128,31 @@ func parseInput(source string) (map[string]*ast.File, error) {
 		}
 	} else {
 		// single file
-		astfile, err := parser.ParseFile(token.NewFileSet(), source, nil, parser.AllErrors)
+		astfile, err := parser.ParseFile(token.NewFileSet(), source, nil, parser.ParseComments)
 		if err != nil {
 			return nil, err
 		}
 		files[source] = astfile
 	}
+
+	for name, f := range files {
+		// check if its a unit test
+		if strings.HasSuffix(name, "_test.go") {
+			delete(files, name)
+			continue
+		}
+
+		// check if its a ssz generated code
+		if comments := f.Comments; len(comments) > 0 {
+			if strings.HasPrefix(comments[0].Text(), "Code generated by fastssz. DO NOT EDIT.") {
+				log.Printf("INFO: Skipped ssz generated object: %v", name)
+
+				delete(files, name)
+				continue
+			}
+		}
+	}
+
 	return files, nil
 }
 
@@ -167,12 +188,44 @@ func (v *Value) isListElem() bool {
 	return strings.HasSuffix(v.name, "]")
 }
 
+func appendWithoutRepeated(s []string, i []string) []string {
+	for _, j := range i {
+		if !contains(j, s) {
+			s = append(s, j)
+		}
+	}
+	return s
+}
+
+func detectImports(v *Value) string {
+	// for sure v is a container
+	var ref string
+	switch v.t {
+	case TypeReference:
+		if !v.noPtr {
+			// it is not a typed reference
+			ref = v.ref
+		}
+	case TypeContainer:
+		ref = v.ref
+	case TypeList, TypeVector:
+		ref = v.e.ref
+	case TypeBytes:
+		ref = v.ref
+	default:
+		ref = v.ref
+	}
+
+	return ref
+}
+
 func (v *Value) objRef() string {
 	// global reference of the object including the package if the reference
 	// is from an external package
 	if v.ref == "" {
 		return v.obj
 	}
+	valuesImported = append(valuesImported, v)
 	return v.ref + "." + v.obj
 }
 
@@ -262,13 +315,14 @@ type env struct {
 	imports []*astImport
 	// excludeTypeNames is a map of type names to leave out of output
 	excludeTypeNames map[string]bool
+	// result is the list of raw astResult objects
+	results []*astResult
+	// suffix is the suffix to append to codec files.
+	suffix string
 }
 
-const encodingPrefix = "_encoding.go"
-
 func (e *env) generateOutputEncodings(output string) (map[string]string, error) {
 	out := map[string]string{}
-
 	keys := make([]string, 0, len(e.order))
 	for k := range e.order {
 		keys = append(keys, k)
@@ -298,7 +352,7 @@ func (e *env) generateEncodings() (map[string]string, error) {
 		// remove .go prefix and replace if with our own
 		ext := filepath.Ext(name)
 		name = strings.TrimSuffix(name, ext)
-		name += encodingPrefix
+		name += e.suffix
 
 		vvv, ok, err := e.print(order)
 		if err != nil {
@@ -312,8 +366,16 @@ func (e *env) generateEncodings() (map[string]string, error) {
 }
 
 func (e *env) hashSource() (string, error) {
+	filenames := []string{}
+	for name := range e.files {
+		filenames = append(filenames, name)
+	}
+	sort.Strings(filenames)
+
 	content := ""
-	for _, f := range e.files {
+	for _, name := range filenames {
+		f := e.files[name]
+
 		var buf bytes.Buffer
 		if err := format.Node(&buf, token.NewFileSet(), f); err != nil {
 			return "", err
@@ -325,6 +387,8 @@ func (e *env) hashSource() (string, error) {
 	return hex.EncodeToString(hash[:]), nil
 }
 
+var valuesImported []*Value
+
 func (e *env) print(order []string) (string, bool, error) {
 	hash, err := e.hashSource()
 	if err != nil {
@@ -362,7 +426,6 @@ func (e *env) print(order []string) (string, bool, error) {
 	}
 
 	objs := []*Obj{}
-	imports := []string{}
 
 	// Print the objects in the order in which they appear on the file.
 	for _, name := range order {
@@ -375,15 +438,13 @@ func (e *env) print(order []string) (string, bool, error) {
 		}
 
 		// detect the imports required to unmarshal this objects
-		refs := detectImports(obj)
-		imports = appendWithoutRepeated(imports, refs)
-
 		if obj.isFixed() && isBasicType(obj) {
 			// we have an alias of a basic type (uint, bool). These objects
 			// will be encoded/decoded inside their parent container and do not
 			// require the sszgen functions.
 			continue
 		}
+
 		objs = append(objs, &Obj{
 			HashTreeRoot: e.hashTreeRoot(name, obj),
 			GetTree:      e.getTree(name, obj),
@@ -398,6 +459,11 @@ func (e *env) print(order []string) (string, bool, error) {
 	}
 	data["objs"] = objs
 
+	imports := []string{}
+	for _, v := range valuesImported {
+		imports = appendWithoutRepeated(imports, []string{detectImports(v)})
+	}
+
 	// insert any required imports
 	importsStr, err := e.buildImports(imports)
 	if err != nil {
@@ -434,41 +500,6 @@ func (e *env) findImport(name string) string {
 	return ""
 }
 
-func appendWithoutRepeated(s []string, i []string) []string {
-	for _, j := range i {
-		if !contains(j, s) {
-			s = append(s, j)
-		}
-	}
-	return s
-}
-
-func detectImports(v *Value) []string {
-	// for sure v is a container
-	// check if any of the fields in the container has an import
-	refs := []string{}
-	for _, i := range v.o {
-		var ref string
-		switch i.t {
-		case TypeReference:
-			if !i.noPtr {
-				// it is not a typed reference
-				ref = i.ref
-			}
-		case TypeContainer:
-			ref = i.ref
-		case TypeList, TypeVector:
-			ref = i.e.ref
-		default:
-			ref = i.ref
-		}
-		if ref != "" {
-			refs = append(refs, ref)
-		}
-	}
-	return refs
-}
-
 // All the generated functions use the '::' string to represent the pointer receiver
 // of the struct method (i.e 'm' in func(m *Method) XX()) for convenience.
 // This function replaces the '::' string with a valid one that corresponds
@@ -499,9 +530,15 @@ func (a *astStruct) isAlias() bool {
 	return a.typ != nil
 }
 
+type aliasRef struct {
+	name  string
+	value uint64
+}
+
 type astResult struct {
 	objs     []*astStruct
 	funcs    []string
+	alias    []*aliasRef
 	packName string
 }
 
@@ -511,6 +548,7 @@ func decodeASTStruct(file *ast.File) *astResult {
 	res := &astResult{
 		objs:     []*astStruct{},
 		funcs:    []string{},
+		alias:    []*aliasRef{},
 		packName: packName,
 	}
 
@@ -536,6 +574,29 @@ func decodeASTStruct(file *ast.File) *astResult {
 					if obj.obj != nil || obj.typ != nil {
 						res.objs = append(res.objs, obj)
 					}
+				} else if valueSpec, ok := spec.(*ast.ValueSpec); ok {
+					// alias value, only consider int numbers. Note that this
+					// only works if the alias is of the type (var Length = 1)
+					// and it will not work for (var Length = uint64(1))
+					names := valueSpec.Names
+					if len(names) != 1 {
+						continue
+					}
+					if vals := valueSpec.Values; len(vals) == 1 {
+						raw := vals[0]
+						if val, ok := raw.(*ast.BasicLit); ok {
+							if val.Kind == token.INT {
+								num, err := strconv.ParseUint(val.Value, 0, 64)
+								if err != nil {
+									panic(fmt.Errorf("BUG: expected int: %v", err))
+								}
+								res.alias = append(res.alias, &aliasRef{
+									name:  names[0].Name,
+									value: num,
+								})
+							}
+						}
+					}
 				}
 			}
 		}
@@ -772,6 +833,7 @@ func (e *env) generateIR() error {
 		}
 	}
 
+	e.results = astResults
 	for _, obj := range e.raw {
 		// If the user does not want to generate a struct we should skip it right away
 		if e.excludeTypeNames[obj.name] {
@@ -808,6 +870,17 @@ func contains(i string, j []string) bool {
 	return false
 }
 
+func (e *env) resolveAlias(name string) (uint64, bool) {
+	for _, i := range e.results {
+		for _, j := range i.alias {
+			if j.name == name {
+				return j.value, true
+			}
+		}
+	}
+	return 0, false
+}
+
 func (e *env) encodeItem(name, tags string) (*Value, error) {
 	v, ok := e.objs[name]
 	if !ok {
@@ -947,94 +1020,106 @@ func (e *env) parseASTFieldType(name, tags string, expr ast.Expr) (*Value, error
 		}
 
 	case *ast.ArrayType:
-		dims, err := extractSSZDimensions(tags)
-		if err != nil {
-			return nil, err
-		}
-
-		collectionExpr := obj
 		outer := &Value{}
-		collection := outer
-		for _, dim := range dims {
-			if dim.IsVector() {
-				collection.t = TypeVector
-				collection.s = uint64(dim.VectorLen())
-			}
-			if dim.IsList() {
-				collection.t = TypeList
-				collection.m = uint64(dim.ListLen())
-				collection.s = uint64(dim.ListLen())
-			}
 
-			// If we're looking at a fixed-size array, attempt to grab the parsed size value. from go/ast
-			// Ellipsis node for [...]T array types, nil for slice types
-			// so when a `[]byte` expression is parsed, Len will be nil:
-			var astSize *uint64
-			// if .Len is nil, this is a slice, not a fixed length array
-			if collectionExpr.Len != nil {
-				arrayLen, ok := collectionExpr.Len.(*ast.BasicLit)
-				if !ok {
-					return nil, fmt.Errorf("failed to parse field %s. byte array definition not understood by go/ast", name)
-				}
-				a, err := strconv.ParseUint(arrayLen.Value, 0, 64)
+		// If we're looking at a fixed-size array, attempt to grab the parsed size value. from go/ast
+		// Ellipsis node for [...]T array types, nil for slice types
+		// so when a `[]byte` expression is parsed, Len will be nil:
+		var astSize *uint64
+		// if .Len is nil, this is a slice, not a fixed length array
+		if obj.Len != nil {
+			switch obj := obj.Len.(type) {
+			case *ast.BasicLit:
+				// fixed array with explicit len
+				a, err := strconv.ParseUint(obj.Value, 0, 64)
 				if err != nil {
 					return nil, fmt.Errorf("could not parse array length for field %s", name)
 				}
 				astSize = &a
-			}
-			if astSize != nil {
-				collection.c = true
-			}
-			if astSize != nil {
-				if collection.t != TypeVector {
-					return nil, fmt.Errorf("unexpected type for fixed size array, name=%s, type=%s", name, collection.t.String())
-				}
-				if collection.s != *astSize {
-					return nil, fmt.Errorf("unexpected mismatch between ssz-size and array fixed size, name=%s, ssz-size=%d, fixed=%d", name, collection.s, *astSize)
-				}
-			}
-
-			switch eeType := collectionExpr.Elt.(type) {
-			case *ast.ArrayType:
-				// we expect there to a subsequent dimension when the element type is an ArrayType
-				// so we update the expression and value container in preparation for the next iteration
-				collectionExpr = eeType
-				collection.e = &Value{}
-				collection = collection.e
-				continue
 			case *ast.Ident:
-				// this condition is preserving the special nesting of byte,
-				// because byte has special handling in the code generator templates.
-				if eeType.Name == "byte" {
-					// note that we are overwriting the list/vector types and replacing them with TypeBytes
-					// TypeBytes can either be a list or vector (determined by looking at the isFixed result)
-					collection.t = TypeBytes
-					if dim.IsVector() {
-						// this is how we differentiate byte lists from byte vectors, rather than the usual approach
-						// of nesting a Value for the element within the .e attribute
-						collection.fixed = true
-					}
-					if dim.IsBitlist() {
-						collection.t = TypeBitList
-					}
-					continue
-				} else {
-					// anything else should recurse to the basic *ast.Ident case defined just below this ArrayType case
-					element, err := e.parseASTFieldType(name, tags, eeType)
-					if err != nil {
-						return nil, err
-					}
-					collection.e = element
+				// fixed array with alias len
+				num, ok := e.resolveAlias(obj.Name)
+				if !ok {
+					return nil, fmt.Errorf("alias ref not found: %s", obj.Name)
 				}
-			default:
-				element, err := e.parseASTFieldType(name, tags, eeType)
+				astSize = &num
+			}
+		}
+		if astSize != nil {
+			outer.c = true
+			outer.s = *astSize
+			outer.fixed = true
+		}
+
+		switch eeType := obj.Elt.(type) {
+		case *ast.Ident:
+			if eeType.Name == "byte" {
+				outer.t = TypeBytes
+			} else {
+				val, err := e.parseASTFieldType(name, "", eeType)
 				if err != nil {
 					return nil, err
 				}
-				collection.e = element
+				outer.e = val
 			}
+		default:
+			element, err := e.parseASTFieldType(name, "", eeType)
+			if err != nil {
+				return nil, err
+			}
+			outer.e = element
 		}
+
+		if tags == "" {
+			// the type was part of a nested array/slice (i.e. []byte of [][]byte).
+			return outer, nil
+		}
+
+		dims, err := extractSSZDimensions(tags)
+		if err != nil {
+			if err == errDimNotFound && outer.isFixed() {
+				// if the item is fixed it does not need dimensions
+				return outer, nil
+			}
+			return nil, fmt.Errorf("%v, tag=%s", err, tags)
+		}
+
+		// try to validate the dimensions
+		outerRef := outer
+		for _, dim := range dims {
+			// upsert the types of the value depending on
+			// what the dimension says
+			if outerRef.t == TypeBytes {
+				if dim.IsVector() {
+					// this is how we differentiate byte lists from byte vectors, rather than the usual approach
+					// of nesting a Value for the element within the .e attribute
+					outerRef.fixed = true
+				}
+				if dim.IsBitlist() {
+					outerRef.t = TypeBitList
+				}
+			} else {
+				if dim.IsVector() {
+					outerRef.t = TypeVector
+				}
+				if dim.IsList() {
+					outerRef.t = TypeList
+				}
+			}
+
+			// update the sizes of the value
+			if dim.IsVector() {
+				outerRef.s = uint64(dim.VectorLen())
+			}
+			if dim.IsList() {
+				outerRef.m = uint64(dim.ListLen())
+				outerRef.s = uint64(dim.ListLen())
+			}
+			outerRef = outerRef.e
+		}
+
 		return outer, nil
+
 	case *ast.Ident:
 		// basic type
 		var v *Value
@@ -1061,10 +1146,10 @@ func (e *env) parseASTFieldType(name, tags string, expr ast.Expr) (*Value, error
 		return v, nil
 
 	case *ast.SelectorExpr:
-		name := obj.X.(*ast.Ident).Name
+		exprName := obj.X.(*ast.Ident).Name
 		sel := obj.Sel.Name
 
-		if name == "time" && sel == "Time" {
+		if exprName == "time" && sel == "Time" {
 			return &Value{t: TypeTime, s: 8}, nil
 		} else if sel == "Bitlist" {
 			// go-bitfield/Bitlist
@@ -1093,7 +1178,7 @@ func (e *env) parseASTFieldType(name, tags string, expr ast.Expr) (*Value, error
 		if err != nil {
 			return nil, fmt.Errorf("failed to encode %s: %v", sel, err)
 		}
-		vv.ref = name
+		vv.ref = exprName
 		vv.noPtr = true
 		return vv, nil
 
@@ -1194,7 +1279,13 @@ func (v *Value) isFixed() bool {
 }
 
 func execTmpl(tpl string, input interface{}) string {
-	tmpl, err := template.New("tmpl").Parse(tpl)
+	funcs := template.FuncMap{
+		"ref": func(v *Value) string {
+			return v.objRef()
+		},
+	}
+
+	tmpl, err := template.New("tmpl").Funcs(funcs).Parse(tpl)
 	if err != nil {
 		panic(err)
 	}
@@ -1207,7 +1298,7 @@ func execTmpl(tpl string, input interface{}) string {
 
 func uintVToName(v *Value) string {
 	if v.t != TypeUint {
-		panic("not expected")
+		panic(fmt.Sprintf("type %v for %s not expected", v.t, v.name))
 	}
 	switch v.s {
 	case 8:
@@ -1222,3 +1313,21 @@ func uintVToName(v *Value) string {
 		panic(fmt.Sprintf("unknown uint size, %d bytes. field name=%s", v.s, v.name))
 	}
 }
+
+func uintVToLowerCaseName(v *Value) string {
+	if v.t != TypeUint {
+		panic(fmt.Sprintf("type %v for %s not expected", v.t, v.name))
+	}
+	switch v.s {
+	case 8:
+		return "uint64"
+	case 4:
+		return "uint32"
+	case 2:
+		return "uint16"
+	case 1:
+		return "uint8"
+	default:
+		panic(fmt.Sprintf("unknown uint size, %d bytes. field name=%s", v.s, v.name))
+	}
+}
